<?php

namespace OPTN;

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://wpxpo.com
 * @since      1.0.0
 *
 * @package    optin
 * @subpackage optin/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    optin
 * @subpackage optin/admin
 */
class Admin {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string $plugin_name The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string $version The current version of this plugin.
	 */
	private $version;


	/**
	 * Initialize the class and set its properties.
	 *
	 * @param string $plugin_name The name of this plugin.
	 * @param string $version The version of this plugin.
	 *
	 * @since    1.0.0
	 */
	public function __construct( $plugin_name, $version ) {
		$this->plugin_name = $plugin_name;
		$this->version     = $version;
	}

	public function register_routes() {
		new \OPTN\AdminRest();
	}

	public function add_menu_page() {

		$menu_icon = base64_encode(
			'<svg xmlns="http://www.w3.org/2000/svg" width="40" height="40" fill="none" viewBox="0 0 40 40"><path  d="M23.8 20a3.8 3.8 0 1 0-7.6 0 3.8 3.8 0 0 0 7.6 0Z"/><path d="M20 39v-7.6A11.4 11.4 0 1 0 8.6 20H1a19 19 0 1 1 19 19Z"/><path d="M16.2 23.8 1 28.8 8 32l3.1 7 5.1-15.2Z"/></svg>'
		);

		add_menu_page(
			__( 'WowOptin', 'optin' ),
			__( 'WowOptin', 'optin' ),
			OPTN_MIN_CAPABILITY,
			'wowoptin',
			'',
			'data:image/svg+xml;base64,' . $menu_icon,
			26
		);

		// Dashboard
		add_submenu_page(
			'wowoptin',
			__( 'WowOptin Dashboard', 'optin' ),
			__( 'Dashboard', 'optin' ),
			OPTN_MIN_CAPABILITY,
			'wowoptin-dashboard',
			function () {
				?>
					<div class="optn-admin-root" id="optn-admin-metrics-app"></div>
				<?php
			},
			0
		);

		// Leads
		add_submenu_page(
			'wowoptin',
			__( 'WowOptin Leads', 'optin' ),
			__( 'Leads', 'optin' ),
			OPTN_MIN_CAPABILITY,
			'wowoptin-leads',
			function () {
				?>
					<div class="optn-admin-root" id="optn-admin-leads-app"></div>
				<?php
			},
			1
		);

		// All conversions page
		add_submenu_page(
			'wowoptin',
			__( 'WowOptin Conversions', 'optin' ),
			__( 'Optins', 'optin' ),
			OPTN_MIN_CAPABILITY,
			'wowoptin-table',
			function () {
				?>
					<div class="optn-admin-root" id="optn-admin-conversions-app"></div>
				<?php
			},
			2
		);

		// Templates page
		add_submenu_page(
			'wowoptin',
			__( 'WowOptin Templates', 'optin' ),
			__( 'Templates', 'optin' ),
			OPTN_MIN_CAPABILITY,
			'wowoptin-templates',
			function () {
				?>
					<div class="optn-admin-root" id="optn-admin-templates-app"></div>
				<?php
			},
			3
		);


		// License Page
		add_submenu_page(
			'wowoptin',
			__( 'WowOptin License', 'optin' ),
			__( 'License', 'optin' ),
			OPTN_MIN_CAPABILITY,
			'wowoptin-license',
			function () {
				?>
					<div class="optn-admin-root" id="optn-admin-license-app"></div>
				<?php
			},
			4
		);

		// Builder Page
		add_submenu_page(
			'wowoptin',
			__( 'WowOptin Builder', 'optin' ),
			__( 'Conversion Builder', 'optin' ),
			OPTN_MIN_CAPABILITY,
			'wowoptin-builder',
			function () {
				?>
					<div class="optn-admin-root" id="optn-admin-builder-app"></div>
				<?php
			}
		);

		remove_submenu_page( 'wowoptin', 'wowoptin' );
	}


	public function add_dropdown_html() {
		?>
			<div id='optn-dropdown' style='display:none;' className='optn-portal'></div>
			<div id='optn-toolbar' style='display:none;'></div>
		<?php
	}


	public function is_in_menu_page() {
		return isset( $_GET['page'] ) && strpos( wp_unslash( $_GET['page'] ), 'wowoptin' ) !== false; // phpcs:ignore
	}

	public function enqueue_styles() {

		// Hides builder menu.
		wp_add_inline_style( 'wp-admin', '#toplevel_page_wowoptin li:has(a[href="admin.php?page=wowoptin-builder"]) {display:none;}' );

		if ( ! $this->is_in_menu_page() ) {
			return;
		}

		wp_enqueue_style( $this->plugin_name . '-admin-css', OPTN_URL . 'admin/css/wowoptin-admin.min.css', array(), $this->version, 'all' );

		wp_enqueue_style( $this->plugin_name . '-admin-build-css', OPTN_URL . '/build/admin/style-index.css', array(), $this->version, 'all' );
	}

	public function enqueue_scripts() {

		if ( ! $this->is_in_menu_page() ) {
			return;
		}

		$assets = require_once OPTN_DIR . '/build/admin/index.asset.php';

		// For media upload feature
		wp_enqueue_media();

		wp_enqueue_script( $this->plugin_name . '-admin-js', OPTN_URL . 'build/admin/index.js', $assets['dependencies'], $assets['version'], true );

		$woo_currency = '';
		if ( function_exists( 'get_woocommerce_currency' ) ) {
			$woo_currency = get_woocommerce_currency();
		}

		$edd_currency = '';

		if ( function_exists( 'edd_currency_get_stored_currency' ) ) {
			$edd_currency = edd_currency_get_stored_currency();
		}

		wp_localize_script(
			$this->plugin_name . '-admin-js',
			'optn_admin',
			array(
				'admin_url'    => admin_url( 'post-new.php' ),
				'template_url' => admin_url( 'admin.php?page=wowoptin-templates' ),
				'builder_url'  => admin_url( 'admin.php?page=wowoptin-builder' ),
				'ajax_url'     => admin_url( 'admin-ajax.php' ),
				'nonce'        => wp_create_nonce( 'optin-nonce' ),
				'asset_url'    => OPTN_URL . 'assets',
				'asset_dir'    => OPTN_DIR . 'assets',
				'server_time'  => date_default_timezone_get(),
				'woo_currency' => $woo_currency,
				'edd_currency' => $edd_currency,
				'curr_user'    => wp_get_current_user()->display_name,
				'dev_mode'     => defined( 'OPTN_DEV_MODE' ) && OPTN_DEV_MODE,
				'pro'          => Helper::is_pro_active() ? 'true' : 'false',
			)
		);
	}
}
