<?php

namespace OPTN;

class DisplayRules {

	const DEFAULT_POST_TYPES = array(
		'post',
		'page',
		'attachment',
		'nav_menu_item',
		'wp_template',
		'wp_template_part',
	);

	public static function get_options() {
		$res = array();

		$post_types = get_post_types(
			array(
				'public'   => true,
				'_builtin' => false,
			),
			'objects'
		);

		$exclude = array( 'product', 'download', 'attachment' );

		if ( ! empty( $post_types ) ) {
			foreach ( $post_types as $post_type ) {

				if ( in_array( $post_type->name, $exclude ) ) {
					continue;
				}

				$res[] = array(
					'isHeader' => true,
					'label'    => ucwords( $post_type->labels->singular_name ),
					'value'    => $post_type->label,
					'pro'      => true,
				);

				$res[] = array(
					'value'   => 'cpt_post:' . $post_type->name,
					'isChild' => true,
					'label'   => $post_type->label,
					'pro'     => true,
				);

				$taxes = get_object_taxonomies( $post_type->name, 'objects' );

				if ( ! empty( $taxes ) ) {
					foreach ( $taxes as $tax ) {
						$res[] = array(
							'value'   => 'cpt_tax:' . $post_type->name . '###' . $tax->name,
							'isChild' => true,
							'label'   => $tax->label,
							'pro'     => true,
						);
					}
				}
			}
		}

		return $res;
	}

	public static function get_values( $type, $search, $exclude ) {

		if ( Helper::check_prefix( $type, 'cpt_post:', true ) ) {
			return self::get_posts( $type, $search, $exclude );
		}

		if ( Helper::check_prefix( $type, 'cpt_tax:', true ) ) {
			$v = explode( '###', $type );
			return self::get_terms( $v[1], $search, $exclude );
		}

		switch ( $type ) {

			case 'post_posts':
				return self::get_posts( 'post', $search, $exclude );

			case 'post_cat':
				return self::get_terms( 'category', $search, $exclude );

			case 'post_tag':
				return self::get_terms( 'post_tag', $search, $exclude );

			case 'page':
				return self::get_posts( 'page', $search, $exclude );

			case 'post_type':
				return self::get_post_types( $search, $exclude );

			case 'author':
				return self::get_users( $search, $exclude );

			case 'woo_cart_products':
				return self::get_woo_products( $search, $exclude );

			case 'edd_cart_products':
				return self::get_edd_products( $search, $exclude );

			default:
				return array();
		}
	}

	public static function get_woo_products( $search, $exclude ) {
		$res = array();

		if ( function_exists( 'wc_get_products' ) ) {
			$products = wc_get_products(
				array(
					'exclude' => $exclude, //phpcs:ignore WordPressVIPMinimum.Performance.WPQueryParams.PostNotIn_exclude
					's'       => $search,
					'limit'   => 10,
					'status'  => 'publish',
				)
			);

			if ( ! empty( $products ) ) {
				foreach ( $products as $product ) {
					$res[] = array(
						'value' => $product->get_id(),
						'label' => $product->get_name(),
					);
				}
			}
		}

		return $res;
	}

	public static function get_edd_products( $search, $exclude ) {
		$res = array();

		if ( function_exists( 'wc_get_products' ) ) {
			$products = get_posts(
				array(
					'post_type'      => 'download',
					'posts_per_page' => 10,
					'post_status'    => 'publish',
					's'              => $search,
					'exclude'        => $exclude, //phpcs:ignore WordPressVIPMinimum.Performance.WPQueryParams.PostNotIn_exclude
				)
			);

			if ( ! empty( $products ) ) {
				foreach ( $products as $product ) {
					$res[] = array(
						'value' => $product->ID,
						'label' => $product->post_title,
					);
				}
			}
		}

		return $res;
	}

	public static function get_terms( $tax, $search, $exclude ) {
		$res = array();

		$terms = get_terms(
			array(
				'taxonomy'   => $tax,
				'hide_empty' => false,
				'exclude'    => $exclude, //phpcs:ignore WordPressVIPMinimum.Performance.WPQueryParams.PostNotIn_exclude
				'search'     => $search,
				'number'     => 10,
			)
		);

		if ( ! empty( $terms ) ) {
			foreach ( $terms as $term ) {
				$res[] = array(
					'label' => $term->name,
					'value' => $term->term_id,
				);
			}
		}

		return $res;
	}

	public static function get_posts( $post_type, $search, $exclude ) {
		$res = array();

		$posts = get_posts(
			array(
				'post_type'      => $post_type,
				's'              => $search,
				'post__not_in'   => $exclude, //phpcs:ignore WordPressVIPMinimum.Performance.WPQueryParams.PostNotIn_post__not_in
				'posts_per_page' => 10,
				'status'         => 'publish',
			)
		);

		if ( ! empty( $posts ) ) {
			foreach ( $posts as $post ) {
				$res[] = array(
					'label' => $post->post_title,
					'value' => $post->ID,
				);
			}
		}

		return $res;
	}

	public static function get_users( $search, $exclude ) {
		$res = array();

		$users = get_users(
			array(
				'search'  => '*' . $search . '*',
				'exclude' => $exclude,
			)
		);

		if ( ! empty( $users ) ) {
			foreach ( $users as $user ) {
				$res[] = array(
					'value' => $user->ID,
					'label' => $user->data->display_name,
				);
			}
		}

		return $res;
	}

	public static function get_post_types( $search, $exclude ) {
		$res     = array();
		$exclude = array_merge(
			$exclude,
			array(
				'product',
				'download',
				'attachment',
				'nav_menu_item',
				'wp_template',
				'wp_template_part',
			)
		);

		$post_types = get_post_types(
			array(
				'public' => true,
			),
			'objects'
		);

		if ( ! empty( $post_types ) ) {
			foreach ( $post_types as $post_type ) {
				if ( ! empty( $search ) ) {
					if ( mb_stripos( $post_type->label, $search ) === false ) {
						continue;
					}
				}

				if ( ! empty( $exclude ) ) {
					if ( in_array( $post_type->name, $exclude ) ) {
						continue;
					}
				}

				$res[] = array(
					'value' => $post_type->name,
					'label' => $post_type->label,
					'pro'   => ! in_array( $post_type->name, self::DEFAULT_POST_TYPES ),
				);
			}
		}

		return $res;
	}
}
