<?php

namespace OPTN;

use OPTN\Helper;
use OPTN\DetectOS;
use OPTN\DetectDevice;
use OPTN\DetectTrafficChannel;
use OPTN\Utils;

require_once ABSPATH . 'wp-admin/includes/upgrade.php';

class DB {
	use \OPTN\Singleton;

	public $CONVERSION_TABLE;
	public $LEAD_TABLE;
	public $INTERACTION_TABLE;
	public $VISITOR_TABLE;
	public $optn_DB_VERSION = 1;
	private $PREFIX         = 'optn_';

	private function __construct() {
		global $wpdb;

		$this->CONVERSION_TABLE  = $wpdb->prefix . $this->PREFIX . 'conversions';
		$this->INTERACTION_TABLE = $wpdb->prefix . $this->PREFIX . 'interactions';
		$this->LEAD_TABLE        = $wpdb->prefix . $this->PREFIX . 'leads';
		$this->VISITOR_TABLE     = $wpdb->prefix . $this->PREFIX . 'visitors';
		update_option( 'optn_db_version', $this->optn_DB_VERSION );
	}

	public function drop_tables() {
		// global $wpdb;
		// $wpdb->query( "DROP TABLE IF EXISTS $this->INTERACTION_TABLE" ); // phpcs:ignore
		// $wpdb->query( "DROP TABLE IF EXISTS $this->LEAD_TABLE" ); // phpcs:ignore
		// $wpdb->query( "DROP TABLE IF EXISTS $this->CONVERSION_TABLE" ); // phpcs:ignore
		// delete_option( 'optn_db_version' );
	}

	public function create_tables() {
		// $this->drop_tables(); // TODO @samin: remove at future versions
		$res = array();

		$res[] = $this->create_conversions_table();
		$res[] = $this->create_interactions_tables();
		$res[] = $this->create_leads_table();

		// Helper::log( $res );
	}


	private function create_conversions_table() {
		global $wpdb;
		$charset_collate = $wpdb->get_charset_collate();

		$table_name = $this->CONVERSION_TABLE;

		return maybe_create_table(
			$table_name,
			"CREATE TABLE $table_name (
				`id` INT UNSIGNED NOT NULL AUTO_INCREMENT,
				`type` TINYINT UNSIGNED NOT NULL,
				`title` VARCHAR(255) NOT NULL,
				`data` LONGTEXT DEFAULT NULL,
				`enabled` BOOLEAN NOT NULL DEFAULT false,
				`end_datetime` TIMESTAMP NULL,
				`traffic_url` VARCHAR(2048) NULL DEFAULT NULL,
				`traffic_channel` MEDIUMINT UNSIGNED DEFAULT NULL,
				`visitor` MEDIUMINT UNSIGNED NOT NULL,
				`device` SMALLINT UNSIGNED NOT NULL,
				`os` MEDIUMINT UNSIGNED NOT NULL,
				`embed` VARCHAR(255) DEFAULT NULL,

				`created_at` TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
				`updated_at` TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,
				`deleted_at` TIMESTAMP NULL DEFAULT NULL,

				PRIMARY KEY (`id`)
			) $charset_collate;"
		);
	}

	private function create_leads_table() {
		global $wpdb;
		$charset_collate = $wpdb->get_charset_collate();

		$table_name = $this->LEAD_TABLE;
		$conv_table = $this->CONVERSION_TABLE;

		return maybe_create_table(
			$table_name,
			"CREATE TABLE $table_name (
				`id` INT UNSIGNED NOT NULL AUTO_INCREMENT,
				`conv_id` INT UNSIGNED DEFAULT NULL,
				`email` VARCHAR(255) DEFAULT NULL,
				`name` VARCHAR(255) DEFAULT NULL,
				`data` JSON DEFAULT NULL,
				`integration` INT UNSIGNED NOT NULL,
				`created_at` TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,

				PRIMARY KEY  (id),
				FOREIGN KEY (`conv_id`) REFERENCES {$conv_table} (`id`) ON DELETE SET NULL,

				INDEX `idx_created_at` (`created_at`)
			) $charset_collate;"
		);
	}

	private function create_interactions_tables() {
		global $wpdb;
		$charset_collate = $wpdb->get_charset_collate();

		$table_name = $this->INTERACTION_TABLE;
		$conv_table = $this->CONVERSION_TABLE;

		return maybe_create_table(
			$table_name,
			"CREATE TABLE $table_name (
				`id` INT UNSIGNED NOT NULL AUTO_INCREMENT,
				`conv_id` INT UNSIGNED NOT NULL,
				`click` BOOLEAN DEFAULT FALSE,
				`order_type` TINYINT UNSIGNED DEFAULT NULL,
				`order_id` INT UNSIGNED DEFAULT NULL,
				`user_id` INT UNSIGNED NULL,
				`ip_address` INT UNSIGNED NULL,
				`country` varchar(3) DEFAULT NULL,
				`path` varchar(255) DEFAULT NULL,
				`device` SMALLINT NULL,
				`created_at` TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
				`updated_at` TIMESTAMP NULL,
				`purchased_at` TIMESTAMP NULL,

				PRIMARY KEY (`id`),
				FOREIGN KEY (`conv_id`) REFERENCES $conv_table (`id`) ON DELETE CASCADE,

				INDEX `idx_created_at` (`created_at`),
				INDEX `idx_updated_at` (`updated_at`)
			) $charset_collate;"
		);
	}

	public function add_lead( $data ) {
		global $wpdb;

		$wpdb->insert( // phpcs:ignore WordPress.DB.DirectDatabaseQuery
			$this->LEAD_TABLE,
			array(
				'conv_id'     => $data['conv_id'],
				'email'       => $data['email'],
				'name'        => $data['name'],
				'data'        => wp_json_encode( $data['data'] ),
				'integration' => $this->get_encoded_integration( $data['integration'] ),
			)
		);
	}

	public function get_leads( $limit, $page ) {
		global $wpdb;

		$offset = ( $page - 1 ) * $limit;

		$sql = $wpdb->prepare(
			"SELECT l.* , c.title AS `conv_title`
			FROM {$this->LEAD_TABLE} l
			LEFT JOIN {$this->CONVERSION_TABLE} c ON c.id = l.conv_id
			ORDER BY l.created_at DESC
			LIMIT %d OFFSET %d",
			$limit,
			$offset
		);

		$raw_data = $wpdb->get_results($sql); // phpcs:ignore

		$res = array();

		foreach ( $raw_data as $d ) {
			$data['id']          = $d->id;
			$data['conv_id']     = ! empty( $d->conv_id ) ? $d->conv_id : null;
			$data['conv_title']  = ! empty( $d->conv_id ) && ! empty( $d->conv_title ) ? $d->conv_title : null;
			$data['email']       = ! empty( $d->email ) ? $d->email : null;
			$data['name']        = ! empty( $d->name ) ? $d->name : null;
			$data['created_at']  = $this->get_formatted_date( $d->created_at );
			$data['data']        = ! empty( $d->data ) ? json_decode( $d->data ) : null;
			$data['integration'] = $this->get_decoded_integration_type( $d->integration );

			$res[] = $data;
		}

		$count = $wpdb->get_var("SELECT COUNT(*) FROM {$this->LEAD_TABLE}"); // phpcs:ignore

		$total = ceil( $count / $limit );

		return array(
			'leads' => $res,
			'total' => $total,
		);
	}

	public function delete_leads( $ids ) {
		global $wpdb;
		$res = null;

		$ids = array_map(
			function ( $id ) {
				return esc_sql( $id );
			},
			$ids
		);

		$ids = implode( ',', $ids );

		$sql = "DELETE FROM {$this->LEAD_TABLE} WHERE `id` IN ( {$ids} )";
		$res = $wpdb->query( $sql ); // phpcs:ignore

		if ( is_int( $res ) && $res < 1 ) {
			return array( 'error' => 404 );
		}

		if ( false === $res ) {
			return array( 'error' => 500 );
		}

		return 200;
	}

	public function get_posts( $status, $limit, $page, $search ) {
		global $wpdb;

		$sql = "SELECT `id`, `title`, `type`, `enabled`, `end_datetime`, `updated_at`, `deleted_at`
					FROM {$this->CONVERSION_TABLE} ";

		$conds = array();

		if ( $status === 'draft' ) {
			$conds[] = 'enabled = 0';
		} elseif ( $status === 'publish' ) {
			$conds[] = 'enabled = 1';
		}

		if ( $status === 'trash' ) {
			$conds[] = 'deleted_at IS NOT NULL';
		} else {
			$conds[] = 'deleted_at IS NULL';
		}

		if ( ! empty( $search ) ) {
			$conds[] = $wpdb->prepare(
				'title LIKE %s ',
				'%' . $wpdb->esc_like( $search ) . '%'
			);
		}

		$sql .= ' WHERE ' . $this->get_cond( $conds );

		$offset = ( $page - 1 ) * $limit;

		$sql .= $wpdb->prepare(
			' ORDER BY `id` DESC LIMIT %d OFFSET %d',
			$limit,
			$offset
		);

		$posts = $wpdb->get_results( // phpcs:ignore WordPress.DB.DirectDatabaseQuery
			$wpdb->prepare( $sql ) // phpcs:ignore WordPress.DB.PreparedSQL.NotPrepared
		);

		foreach ( $posts as $post ) {
			$post->type       = $this->get_decoded_conv_type( $post->type );
			$post->updated_at = $this->get_formatted_date( $post->updated_at );
			$post->enabled    = $post->enabled ? true : false;

			if ( ! empty( $post->end_datetime ) ) {
				$post->end_datetime = $this->get_formatted_date( $post->end_datetime );
			}

			if ( ! empty( $post->deleted_at ) ) {
				$post->deleted_at = $this->get_formatted_date( $post->deleted_at );
			}
		}

		$count       = $this->get_post_status_count( $search );
		$total_pages = ceil( $count[ $status ] / $limit );

		return array(
			'posts'        => $posts,
			'count'        => $count,
			'total_pages'  => $total_pages,
			'current_page' => $page,
		);
	}

	public function get_post_for_select() {
		global $wpdb;

		return $wpdb->get_results( // phpcs:ignore WordPress.DB.DirectDatabaseQuery
			$wpdb->prepare(
				"SELECT c.`id` as `value`, c.`title` as `label`
				FROM {$this->CONVERSION_TABLE} c", // phpcs:ignore WordPress.DB.PreparedSQL.InterpolatedNotPrepared
			)
		);
	}

	public function get_impressions( $id, $days, $unique ) {
		global $wpdb;

		$sql = 'SELECT DATE(i.`created_at`) AS `date`, COUNT(*) as `t_views`,  COUNT(DISTINCT i.`ip_address`) as `u_views`';

		$sql .= " FROM {$this->INTERACTION_TABLE} i";

		$conds = array(
			$wpdb->prepare(
				'i.`created_at` >= (NOW() - INTERVAL %d DAY)',
				$days
			),
		);

		if ( ! empty( $id ) ) {
			$conds[] = $wpdb->prepare( 'i.`conv_id` = %d', $id );
		}

		$sql .= ' WHERE ' . $this->get_cond( $conds );

		$sql .= ' GROUP BY DATE(i.`created_at`) ORDER BY DATE(i.`created_at`) DESC';

		$raw_data = $wpdb->get_results( $wpdb->prepare( $sql ) ); // phpcs:ignore

		$data = array();

		foreach ( $raw_data as $d ) {
			$data[ $d->date ] = array(
				'total'  => $d->t_views,
				'unique' => $d->u_views,
			);
		}

		$res = array(
			'total'  => array(),
			'unique' => array(),
		);

		$periods = Helper::get_past_dates( $days );

		foreach ( $periods as $period ) {
			$res ['total'] [ $period ]  = isset( $data[ $period ]['total'] ) ? intval( $data[ $period ]['total'] ) : 0;
			$res ['unique'] [ $period ] = isset( $data[ $period ]['unique'] ) ? intval( $data[ $period ]['unique'] ) : 0;
		}

		return $res;
	}

	public function get_impressions_by_device( $id, $days ) {

		$device_map = array(
			__( 'Desktop', 'optin' ),
			__( 'Tablet', 'optin' ),
			__( 'Mobile', 'optin' ),
		);

		global $wpdb;

		$sql = 'SELECT i.`device` AS `device`, COUNT(*) AS `views`';

		$sql .= " FROM {$this->INTERACTION_TABLE} i";

		$conds = array(
			$wpdb->prepare(
				'i.`created_at` >= (NOW() - INTERVAL %d DAY)',
				$days
			),
		);

		if ( ! empty( $id ) ) {
			$conds[] = $wpdb->prepare( 'i.`conv_id` = %d', $id );
		}

		$sql .= ' WHERE ' . $this->get_cond( $conds );

		$sql .= ' GROUP BY i.`device` ORDER BY i.`device`';

		$data = $wpdb->get_results( $sql ); // phpcs:ignore

		$device_data = array();

		foreach ( $data as $d ) {
			$device_data[ $d->device ] = $d->views;
		}

		$res   = array();
		$total = array_sum( array_column( $data, 'views' ) );

		for ( $i = 0; $i < count( $device_map ); $i++ ) {
			$res[] = array(
				'label' => $device_map[ $i ],
				'value' => isset( $device_data[ $i ] ) ? intval( $device_data[ $i ] ) : 0,
				'pct'   => isset( $device_data[ $i ] ) ? round( ( $device_data[ $i ] / $total ) * 100, 2 ) : 0,
			);
		}

		return $res;
	}

	public function get_conversions( $id, $days ) {
		global $wpdb;

		$conds = array();

		if ( ! empty( $id ) ) {
			$conds[] = $wpdb->prepare( 'i.`conv_id` = %d', $id );
		}

		$cond_str = '';

		if ( ! empty( $conds ) ) {
			$cond_str = ' AND ' . $this->get_cond( $conds );
		}

		$raw_click_sql =
			"SELECT DATE(i.`updated_at`) AS `date`, COUNT(*) AS `total` 
			FROM {$this->INTERACTION_TABLE} i
			WHERE i.`click` = TRUE AND
				DATE(i.`updated_at`) >= DATE(NOW() - INTERVAL %d DAY) {$cond_str}
			GROUP BY DATE(i.`updated_at`)
			ORDER BY DATE(i.`updated_at`) DESC";

		$raw_click_data = $wpdb->get_results( $wpdb->prepare( $raw_click_sql, $days ) ); // phpcs:ignore

		$raw_sales_sql =
			"SELECT DATE(i.`purchased_at`) AS `date`, COUNT(*) AS `total`
			FROM {$this->INTERACTION_TABLE} i
				WHERE
				i.order_id IS NOT NULL AND
				DATE(i.`purchased_at`) >= DATE(NOW() - INTERVAL %d DAY) {$cond_str}
			GROUP BY DATE(i.`purchased_at`)
			ORDER BY DATE(i.`purchased_at`) DESC";

		$raw_sales_data = $wpdb->get_results( $wpdb->prepare( $raw_sales_sql, $days )); //phpcs:ignore

		$click = array();
		$sales = array();

		foreach ( $raw_click_data as $c ) {
			$click[ $c->date ] = $c->total;
		}

		foreach ( $raw_sales_data as $s ) {
			$sales[ $s->date ] = $s->total;
		}

		$periods = Helper::get_past_dates( $days );
		$res     = array();

		foreach ( $periods as $period ) {
			$res[ $period ] =
				( isset( $click[ $period ] ) ? intval( $click[ $period ] ) : 0 ) +
				( isset( $sales[ $period ] ) ? intval( $sales[ $period ] ) : 0 );
		}

		return $res;
	}

	public function get_sales( $id, $days ) {
		global $wpdb;

		$conds = array();

		if ( ! empty( $id ) ) {
			$conds[] = "i.`conv_id` = {$id}";
		}

		$cond_str = '';

		if ( ! empty( $conds ) ) {
			$cond_str = ' AND ' . $this->get_cond( $conds );
		}

		$raw_data = $wpdb->get_results( // phpcs:ignore WordPress.DB.DirectDatabaseQuery
			$wpdb->prepare(
				'SELECT DATE(i.`purchased_at`) AS `date`, i.`order_type`, i.`order_id` ' .
				" FROM {$this->INTERACTION_TABLE} i " . // phpcs:ignore WordPress.DB.PreparedSQL.InterpolatedNotPrepared
				" WHERE i.`order_id` IS NOT NULL AND i.`purchased_at` IS NOT NULL AND DATE(i.`purchased_at`) >= DATE(NOW() - INTERVAL %d DAY) {$cond_str} GROUP BY DATE(i.`purchased_at`) ORDER BY DATE(i.`purchased_at`) DESC", // phpcs:ignore WordPress.DB.PreparedSQL.InterpolatedNotPrepared
				$days,
			)
		);

		$data = array();

		foreach ( $raw_data as $c ) {
			$data[ $c->date ] = array(
				'order_type' => $c->order_type,
				'order_id'   => $c->order_id,
			);
		}

		$periods = Helper::get_past_dates( $days );
		$res     = array();

		foreach ( $periods as $period ) {
			$res[ $period ] = isset( $data[ $period ] ) ? floatval( Helper::get_revenue( $data[ $period ]['order_type'], $data[ $period ]['order_id'] ) ) : 0.00;
		}

		return $res;
	}

	public function get_pop_optin( $days, $limit, $page ) {
		global $wpdb;

		$offset = ( $page - 1 ) * $limit;

		$raw_data = $wpdb->get_results( // phpcs:ignore WordPress.DB.DirectDatabaseQuery
			$wpdb->prepare(
				'SELECT c.id, c.title, COUNT(i.id) AS `views` ' .
				" FROM {$this->CONVERSION_TABLE} c " . // phpcs:ignore WordPress.DB.PreparedSQL.InterpolatedNotPrepared
				" LEFT JOIN {$this->INTERACTION_TABLE} i ON " . // phpcs:ignore WordPress.DB.PreparedSQL.InterpolatedNotPrepared
					'c.id = i.conv_id AND
					DATE(i.created_at) >= DATE(NOW() - INTERVAL %d DAY)
				WHERE
					c.deleted_at IS NULL
				GROUP BY c.id
				ORDER BY `views` DESC
				LIMIT %d OFFSET %d',
				$days,
				$limit,
				$offset
			)
		);

		$raw_conv_data = $wpdb->get_results( // phpcs:ignore WordPress.DB.DirectDatabaseQuery
			$wpdb->prepare(
				'SELECT i.conv_id, COUNT(*) AS `conv` ' .
				" FROM {$this->INTERACTION_TABLE} i " . // phpcs:ignore WordPress.DB.PreparedSQL.InterpolatedNotPrepared
				' WHERE
					(i.click = 1 OR i.order_id IS NOT NULL) AND
					DATE(i.updated_at) >= DATE(NOW() - INTERVAL %d DAY)
				GROUP BY i.conv_id',
				$days
			)
		);

		$conv = array();

		foreach ( $raw_conv_data as $c ) {
			$conv[ $c->conv_id ] = $c->conv;
		}

		$res = array();

		foreach ( $raw_data as $c ) {
			$cr            = isset( $conv[ $c->id ] ) ? round( ( $conv[ $c->id ] / $c->views ) * 100, 2 ) : 0;
			$res['data'][] = array(
				'title' => $c->title,
				'views' => $c->views,
				'cr'    => $cr,
			);
		}

		$total = $wpdb->get_row( // phpcs:ignore WordPress.DB.DirectDatabaseQuery
			$wpdb->prepare(
				'SELECT COUNT(*) AS `count` ' .
				" FROM {$this->CONVERSION_TABLE} c " . // phpcs:ignore WordPress.DB.PreparedSQL.InterpolatedNotPrepared
				' WHERE c.`deleted_at` IS NULL'
			)
		);

		$res['page']['total'] = ceil( intval( $total->count ) / $limit );

		return $res;
	}

	public function get_geo_view_data( $days ) {
		global $wpdb;
		$res = array(
			'dom' => 0,
			'map' => array(),
			'pct' => array(),
		);

		$raw_data = $wpdb->get_results( // phpcs:ignore WordPress.DB.DirectDatabaseQuery
			$wpdb->prepare(
				"SELECT LOWER(COALESCE(i.country, 'unknown')) AS `country`, COUNT(i.country) as `value` " .
				" FROM {$this->INTERACTION_TABLE} i " . // phpcs:ignore WordPress.DB.PreparedSQL.InterpolatedNotPrepared
				' WHERE
					DATE(i.created_at) >= DATE(NOW() - INTERVAL %d DAY)
				GROUP BY i.country
				ORDER BY `value` DESC',
				$days
			)
		);

		$count      = count( $raw_data );
		$res['dom'] = \OPTN\CountryDecoder::get_dom_pct( $count );

		$res['map'] = $raw_data;

		$top5  = array_slice( $raw_data, 0, 5 );
		$total = array_sum( array_column( $top5, 'value' ) );

		$unk = 0;

		foreach ( $top5 as $d ) {
			$name = \OPTN\CountryDecoder::decode( $d->country );
			$pct  = ! empty( $total ) ? round( ( $d->value / $total ) * 100, 2 ) : 0;
			$unk += empty( $name ) ? $pct : 0;

			if ( ! empty( $name ) ) {
				$res['pct'][] = array(
					'name' => $name,
					'pct'  => $pct,
				);
			}
		}

		if ( $unk > 0 ) {
			$res['pct'][] = array(
				'name' => __( 'Unknown', 'optin' ),
				'pct'  => round( $unk, 2 ),
			);
		}

		return $res;
	}

	private function get_post_status_count( $search ) {
		global $wpdb;
		$res = array();

		$cond = '';

		if ( ! empty( $search ) ) {
			$cond = $wpdb->prepare( ' AND c.title LIKE %s', '%' . $wpdb->esc_like( $search ) . '%' );
		}

		$sql = "SELECT
					CASE
						WHEN c.enabled = 0 THEN 'draft'
						WHEN c.enabled = 1 THEN 'publish'
					END as `status`,
					COUNT(*) AS `count` 
				FROM {$this->CONVERSION_TABLE} c
				WHERE c.deleted_at IS NULL {$cond}
				GROUP BY enabled

				UNION

				SELECT
					CASE
						WHEN true THEN 'trash'
					END AS `status`, 		
					COUNT(*) AS `count` 
				FROM {$this->CONVERSION_TABLE} c 
				WHERE c.deleted_at IS NOT NULL ${cond}
		";

		$data = $wpdb->get_results( $sql ); // phpcs:ignore

		foreach ( $data as $d ) {
			$res[ $d->status ] = $d->count;
		}

		$statues = array( 'draft', 'publish', 'trash' );
		foreach ( $statues as $status ) {
			if ( ! isset( $res[ $status ] ) ) {
				$res[ $status ] = '0';
			}
		}

		$res['all'] = strval( $res['draft'] + $res['publish'] );

		return $res;
	}

	public function add_view( $id ) {
		global $wpdb;

		$curr_user_id = get_current_user_id();
		$curr_user_id = $curr_user_id === 0 ? null : $curr_user_id;

		$user_info = Utils::get_user_info();
		$ip        = ! empty( $user_info['ip'] ) ? ip2long( $user_info['ip'] ) : null;

		$device = DetectDevice::get_device();
		$device = $device === 'lg' ? 0 : ( $device === 'sm' ? 1 : 2 );

		$wpdb->insert( // phpcs:ignore WordPress.DB.DirectDatabaseQuery
			$this->INTERACTION_TABLE,
			array(
				'conv_id'    => $id,
				'user_id'    => $curr_user_id,
				'ip_address' => $ip,
				'country'    => $user_info['country'],
				'device'     => $device,
			),
		);

		$curr_id = $wpdb->insert_id;

		return array(
			'id' => $curr_id,
		);
	}

	public function add_goal( $id, $goal_type, $value, $path ) {
		global $wpdb;

		$res = $wpdb->update( // phpcs:ignore WordPress.DB.DirectDatabaseQuery
			$this->INTERACTION_TABLE,
			array(
				'click'      => true,
				'path'       => $path,
				'updated_at' => current_time( 'mysql' ),
			),
			array(
				'id' => $id,
			)
		);

		return array(
			'success' => is_int( $res ) && $res > 0 ? 1 : 0,
		);
	}

	public function add_purchase( $id, $conv_id, $order_type, $order_id ) {
		$type = null;

		switch ( $order_type ) {
			case 'woo':
				$type = 0;
				break;
			case 'edd':
				$type = 1;
				break;
			default:
				return 0;
		}

		global $wpdb;

		$res = $wpdb->update( // phpcs:ignore WordPress.DB.DirectDatabaseQuery
			$this->INTERACTION_TABLE,
			array(
				'order_type'   => $type,
				'order_id'     => $order_id,
				'purchased_at' => current_time( 'mysql' ),
			),
			array(
				'id'      => $id,
				'conv_id' => $conv_id,
			)
		);

		return array(
			'success' => is_int( $res ) && $res > 0 ? 1 : 0,
		);
	}

	public function get_quick_view_data( $id, $days ) {
		global $wpdb;
		$res = array();

		$id_cond = ( ! empty( $id ) ? $wpdb->prepare( ' AND i.`conv_id` = %d ', $id ) : '' );

		$curr_data_sql = "SELECT COUNT(*) AS `total`
							FROM {$this->INTERACTION_TABLE} i
							WHERE DATE(i.created_at) >= DATE(NOW() - INTERVAL 7 DAY) " . $id_cond;

		$curr_data = $wpdb->get_row( $wpdb->prepare( $curr_data_sql, $id_cond ) ); // phpcs:ignore

		$old_data = $wpdb->get_row( // phpcs:ignore WordPress.DB.DirectDatabaseQuery
			$wpdb->prepare(
				'SELECT COUNT(*) AS `total` ' .
				" FROM {$this->INTERACTION_TABLE} i " . // phpcs:ignore WordPress.DB.PreparedSQL.InterpolatedNotPrepared
				' WHERE
					DATE(i.created_at) < DATE(NOW() - INTERVAL 7 DAY) AND
					DATE(i.created_at) >= DATE(NOW() - INTERVAL 14 DAY) ' . $id_cond, // phpcs:ignore WordPress.DB.PreparedSQL.NotPrepared
			)
		);

		$res['view']['total'] = Helper::format_number( $curr_data->total );
		$res['view']['diff']  = Helper::get_diff_pct( $old_data->total, $curr_data->total );

		$click = $wpdb->get_results( // phpcs:ignore WordPress.DB.DirectDatabaseQuery
			// phpcs:ignore WordPress.DB.PreparedSQLPlaceholders.MissingReplacements
			$wpdb->prepare(
				$wpdb->prepare(
					'SELECT COUNT(*) AS `total` ' .
					" FROM {$this->INTERACTION_TABLE} i " . // phpcs:ignore WordPress.DB.PreparedSQL.InterpolatedNotPrepared
					' WHERE
						i.click = TRUE AND
						DATE(i.created_at) >= DATE(NOW() - INTERVAL 7 DAY) ' . $id_cond, // phpcs:ignore WordPress.DB.PreparedSQL.NotPrepared
				)

				. ' UNION ' .

				$wpdb->prepare(
					'SELECT COUNT(*) AS `total` ' .
					" FROM {$this->INTERACTION_TABLE} i " . // phpcs:ignore WordPress.DB.PreparedSQL.InterpolatedNotPrepared
					' WHERE
						i.click = TRUE AND
						DATE(i.created_at) < DATE(NOW() - INTERVAL 7 DAY) AND
						DATE(i.created_at) >= DATE(NOW() - INTERVAL 14 DAY) ' . $id_cond, // phpcs:ignore WordPress.DB.PreparedSQL.NotPrepared
				)
			)
		);

		$sales = $wpdb->get_results( // phpcs:ignore WordPress.DB.DirectDatabaseQuery
			// phpcs:ignore WordPress.DB.PreparedSQLPlaceholders.MissingReplacements
			$wpdb->prepare(
				$wpdb->prepare(
					'SELECT COUNT(*) AS `total` ' .
					" FROM {$this->INTERACTION_TABLE} i " . // phpcs:ignore WordPress.DB.PreparedSQL.InterpolatedNotPrepared
					' WHERE
						i.order_id IS NOT NULL AND
						DATE(i.created_at) >= DATE(NOW() - INTERVAL 7 DAY) ' . $id_cond, // phpcs:ignore WordPress.DB.PreparedSQL.NotPrepared
				)

				. ' UNION ' .

				$wpdb->prepare(
					'SELECT COUNT(*) AS `total` ' .
					" FROM {$this->INTERACTION_TABLE} i " . // phpcs:ignore WordPress.DB.PreparedSQL.InterpolatedNotPrepared
					' WHERE
						i.order_id IS NOT NULL AND
						DATE(i.created_at) < DATE(NOW() - INTERVAL 7 DAY) AND
						DATE(i.created_at) >= DATE(NOW() - INTERVAL 14 DAY) ' . $id_cond, // phpcs:ignore WordPress.DB.PreparedSQL.NotPrepared
				)
			)
		);

		$curr = $click[0]->total + $sales[0]->total;
		$old  = ( isset( $click[1]->total ) ? intval( $click[1]->total ) : 0 ) + ( isset( $sales[1]->total ) ? intval( $sales[1]->total ) : 0 );

		$res['conv']['total'] = Helper::format_number( $curr );
		$res['conv']['diff']  = Helper::get_diff_pct( $old, $curr );

		$views = $wpdb->get_results( // phpcs:ignore WordPress.DB.DirectDatabaseQuery
			// phpcs:ignore WordPress.DB.PreparedSQLPlaceholders.MissingReplacements
			$wpdb->prepare(
				$wpdb->prepare(
					'SELECT COUNT(*) AS `total` ' .
					" FROM {$this->INTERACTION_TABLE} i " . // phpcs:ignore WordPress.DB.PreparedSQL.InterpolatedNotPrepared
					' WHERE DATE(i.created_at) >= DATE(NOW() - INTERVAL 7 DAY) ' . $id_cond, // phpcs:ignore WordPress.DB.PreparedSQL.NotPrepared
				)

				. ' UNION ' .

				$wpdb->prepare(
					'SELECT COUNT(*) AS `total` ' .
					" FROM {$this->INTERACTION_TABLE} i " . // phpcs:ignore WordPress.DB.PreparedSQL.InterpolatedNotPrepared
					' WHERE
						DATE(i.created_at) < DATE(NOW() - INTERVAL 7 DAY) AND
						DATE(i.created_at) >= DATE(NOW() - INTERVAL 14 DAY) ' . $id_cond, // phpcs:ignore WordPress.DB.PreparedSQL.NotPrepared
				)
			)
		);

		$curr = ! empty( $views[0]->total ) ? ( ( $click[0]->total ?? 0 ) + ($sales[0]->total ?? 0) ) / $views[0]->total : 0;
		$old  = ! empty( $views[1]->total ) ? ( ( $click[1]->total ?? 0 ) + ($sales[1]->total ?? 0) ) / $views[1]->total : 0;

		$res['rate']['total'] = strval( round( $curr, 2 ) ) . '%';
		$res['rate']['diff']  = Helper::get_diff_pct( $old, $curr );

		$curr_sales = $wpdb->get_results( // phpcs:ignore WordPress.DB.DirectDatabaseQuery
			$wpdb->prepare(
				'SELECT i.order_type, i.order_id ' .
				" FROM {$this->INTERACTION_TABLE} i " . // phpcs:ignore WordPress.DB.PreparedSQL.InterpolatedNotPrepared
				' WHERE
					i.order_id IS NOT NULL AND
					DATE(i.created_at) >= DATE(NOW() - INTERVAL 7 DAY) ' . $id_cond, // phpcs:ignore WordPress.DB.PreparedSQL.NotPrepared
			)
		);

		$old_sales = $wpdb->get_results( // phpcs:ignore WordPress.DB.DirectDatabaseQuery
			$wpdb->prepare(
				'SELECT i.order_type, i.order_id ' .
				" FROM {$this->INTERACTION_TABLE} i " . // phpcs:ignore WordPress.DB.PreparedSQL.InterpolatedNotPrepared
				' WHERE
					i.order_id IS NOT NULL AND
					DATE(i.created_at) < DATE(NOW() - INTERVAL 7 DAY) AND
					DATE(i.created_at) >= DATE(NOW() - INTERVAL 14 DAY) ' . $id_cond, // phpcs:ignore WordPress.DB.PreparedSQL.NotPrepared
			)
		);

		$curr = 0;
		$old  = 0;

		foreach ( $curr_sales as $sale ) {
			$curr += Helper::get_revenue( $sale->order_type, $sale->order_id );
		}

		foreach ( $old_sales as $sale ) {
			$old += Helper::get_revenue( $sale->order_type, $sale->order_id );
		}

		$total = strval( Helper::format_number( $curr ) );

		if ( function_exists( 'get_woocommerce_currency_symbol' ) ) {
			$total = trim( html_entity_decode( get_woocommerce_currency_symbol() ) ) . $total;
		} elseif ( function_exists( 'edd_currency_filter' ) ) {
			$total = html_entity_decode( edd_currency_filter( $total ) );
		}

		$res['rev']['total'] = $total;
		$res['rev']['diff']  = Helper::get_diff_pct( $old, $curr );

		return $res;
	}

	private function get_processed_data_for_db( $post_data, $type ) {

		$data = array();

		if ( $post_data->audience->tSrc === 'src' ) {
			$traffic_url         = isset( $post_data->audience->tUrl ) ? $post_data->audience->tUrl : null;
			$data['traffic_url'] = $traffic_url;
		} else {
			$data['traffic_url'] = null;
		}

		if ( $post_data->schedule->endType === 'custom' && ! empty( $post_data->schedule->endDate ) && ! empty( $post_data->schedule->endTime ) ) {
			$end_datetime         = gmdate( 'Y-m-d H:i:s', strtotime( $post_data->schedule->endDate . ' ' . $post_data->schedule->endTime ) );
			$data['end_datetime'] = $end_datetime;
		} else {
			$data['end_datetime'] = null;
		}

		$data['device']          = $this->get_encoded_device( $post_data->audience );
		$data['os']              = $this->get_encoded_os( $post_data->audience );
		$data['traffic_channel'] = $this->get_encoded_traffic_channel( $post_data->audience );
		$data['visitor']         = $this->get_encoded_visitor( $post_data->audience );

		$data['embed'] = null;

		if ( 'inline' === $type && 'embed' === $post_data->design->pos->inlineType && ! empty( $post_data->design->pos->embedPosition ) ) {
			$hook = sanitize_key( $post_data->design->pos->embedPosition );

			if ( 'custom' === $hook ) {
				$hook = 'custom###' .
					sanitize_key( $post_data->design->pos->customHookPos ) . '###' .
					sanitize_key( $post_data->design->pos->customHookName );
			}

			$data['embed'] = esc_sql( $hook );
		}

		return $data;
	}

	public function update_saved_post( $id, $title, $status, $data, $type ) {
		if ( empty( $id ) ) {
			return array( 'error' => 'Invalid Post ID' );
		}

		global $wpdb;

		$post_data = empty( $data ) ? null : json_decode( $data );

		if ( ! empty( $post_data ) ) {
			$updated_data         = $this->get_processed_data_for_db( $post_data, $type );
			$updated_data['data'] = $data;
		} else {
			$updated_data = array();
		}

		if ( ! empty( $title ) ) {
			$updated_data['title'] = $title;
		}

		$updated_data['enabled'] = is_bool( $status ) ? $status : false;

		if ( $updated_data['enabled'] ) {
			$updated_data['deleted_at'] = null;
		}

		$res = $wpdb->update( // phpcs:ignore WordPress.DB.DirectDatabaseQuery
			$this->CONVERSION_TABLE,
			$updated_data,
			array( 'id' => $id )
		);

		return array( $res );
	}

	public function get_saved_post( $id ) {
		$conversion = $this->get_conv_by_id( $id );

		if ( ! empty( $conversion ) ) {
			return array(
				'title'  => $conversion->title,
				'data'   => isset( $conversion->data ) ? $conversion->data : null,
				'status' => $conversion->enabled ? 'publish' : 'draft',
			);
		}

		return null;
	}

	public function get_conv_by_id( $id ) {
		global $wpdb;
		return $wpdb->get_row( $wpdb->prepare( "SELECT * FROM {$this->CONVERSION_TABLE} WHERE `id` = %d", $id ) ); // phpcs:ignore
	}

	private function copy_post( $id ) {
			global $wpdb;

			$original = $this->get_conv_by_id( $id );

			$title = $original->title . ' - Copy';

			$data = array(
				'title'           => $title,
				'enabled'         => false,
				'data'            => $original->data,
				'type'            => $original->type,
				'end_datetime'    => $original->end_datetime,
				'traffic_url'     => $original->traffic_url,
				'traffic_channel' => $original->traffic_channel,
				'visitor'         => $original->visitor,
				'device'          => $original->device,
				'os'              => $original->os,
				'embed'           => $original->embed,
			);

			$res = $wpdb->insert( $this->CONVERSION_TABLE, $data ); // phpcs:ignore WordPress.DB.DirectDatabaseQuery

			$id = is_int( $res ) && $res > 0 ? $wpdb->insert_id : null;

			return array(
				'id'     => $id,
				'title'  => $title,
				'data'   => $original->data,
				'status' => 'draft',
			);
	}

	public function create_new_post( $type, $copy, $title, $template_id ) {
		global $wpdb;

		if ( ! empty( $copy ) ) {
			return $this->copy_post( $copy );
		} elseif ( ! empty( $template_id ) ) {
			$data = \OPTN\Templates::fetch_single_template( $template_id );

			if ( empty( $data ) ) {
				return null;
			}

			$enc_data = wp_json_encode($data);

			$db_data            = $this->get_processed_data_for_db( $data, $type );
			$db_data['title']   = sanitize_text_field( $title );
			$db_data['enabled'] = false;
			$db_data['data']    = $enc_data;
			$db_data['type']    = $this->get_encoded_conv_type( $type );

			$res = $wpdb->insert( $this->CONVERSION_TABLE, $db_data ); // phpcs:ignore WordPress.DB.DirectDatabaseQuery

			$id = is_int( $res ) && $res > 0 ? $wpdb->insert_id : null;

			return array(
				'id'     => $id,
				'title'  => $title,
				'data'   => $enc_data,
				'status' => 'draft',
			);
		} else {
			$title = ! empty( $title ) ? sanitize_text_field( $title ) : __( 'New Optin', 'optin' ) . ' ' . current_time( 'Y-m-d H:i:s' );

			$data = array(
				'title'           => $title,
				'type'            => $this->get_encoded_conv_type( $type ),
				'traffic_channel' => $this->get_encoded_traffic_channel( 'default' ),
				'visitor'         => $this->get_encoded_visitor( 'default' ),
				'device'          => $this->get_encoded_device( 'default' ),
				'os'              => $this->get_encoded_os( 'default' ),
			);

			$res = $wpdb->insert( $this->CONVERSION_TABLE, $data ); // phpcs:ignore WordPress.DB.DirectDatabaseQuery

			$id = is_int( $res ) && $res > 0 ? $wpdb->insert_id : null;

			return array(
				'id'    => $id,
				'title' => $title,
			);
		}
	}

	public function delete_post( $ids, $type ) {
		global $wpdb;
		$res = null;

		$ids = array_map(
			function ( $id ) {
				return esc_sql( $id );
			},
			$ids
		);

		$ids = implode( ',', $ids );

		if ( $type === 'soft' ) {
			$sql = "UPDATE {$this->CONVERSION_TABLE}  
				 	SET `deleted_at` = NOW(), `enabled` = 0
					WHERE `id` IN ( {$ids} )";
			$res = $wpdb->query( $sql ); // phpcs:ignore
		} elseif ( $type === 'hard' ) {
			$sql = "DELETE FROM {$this->CONVERSION_TABLE} WHERE `id` IN ( {$ids} )";
			$res = $wpdb->query( $sql ); // phpcs:ignore
		}

		if ( is_int( $res ) && $res < 1 ) {
			return array( 'error' => 404 );
		}

		if ( false === $res ) {
			return array( 'error' => 500 );
		}

		return 200;
	}

	/**
	 * Restores a optin from trash
	 *
	 * @param int[] $ids
	 * @return int
	 */
	public function restore_post( $ids ) {
		global $wpdb;

		$ids = array_map(
			function ( $id ) {
				return esc_sql( $id );
			},
			$ids
		);

		$ids = implode( ',', $ids );

		$sql = "UPDATE {$this->CONVERSION_TABLE}
				SET `deleted_at` = NULL
				WHERE `id` IN ( {$ids} )";

		$res = $wpdb->query( $sql ); // phpcs:ignore

		return $res;
	}

	public function get_conv_for_render( $id = null, $embed = false ) {
		$device           = $this->get_encoded_device( DetectDevice::get_device() );
		$os               = $this->get_encoded_os( DetectOS::getOS() );
		$visitor_types    = $this->get_encoded_visitor( Utils::get_visitor_types() );
		$traffic_channels = $this->get_encoded_traffic_channel( DetectTrafficChannel::get_traffic_channels() );

		global $wpdb;

		$id_cond = '';

		if ( ! empty( $id ) ) {
			$id_cond .= $wpdb->prepare( ' AND c.id = %d ', $id );
		}

		if ( ! empty( $embed ) ) {
			$id_cond .= ' AND c.embed IS NOT NULL ';
		} else {
			$id_cond .= ' AND c.embed IS NULL ';
		}

		$posts = $wpdb->get_results( // phpcs:ignore WordPress.DB.DirectDatabaseQuery
			$wpdb->prepare(
				' SELECT * ' .
				" FROM {$this->CONVERSION_TABLE} c " . // phpcs:ignore WordPress.DB.PreparedSQL.InterpolatedNotPrepared
				" WHERE
					c.enabled = 1 AND
					c.device & %d > 0 AND
					c.os & %d > 0 AND
					c.visitor & %d > 0 AND
					c.traffic_channel & %d > 0 AND
					( c.end_datetime IS NULL OR c.end_datetime > NOW() ) AND
					( c.traffic_url IS NULL OR c.traffic_url = %s )
					{$id_cond}
				",
				$device,
				$os,
				$visitor_types,
				$traffic_channels,
				isset( $_SERVER['HTTP_REFERER'] ) ? sanitize_text_field( wp_unslash( $_SERVER['HTTP_REFERER'] ) ) : ''
			)
		);

		return $posts;
	}

	private function get_encoded_conv_type( $type ) {
		switch ( $type ) {
			case 'popup':
				return 0;
			case 'banner':
				return 1;
			case 'slidein':
				return 2;
			case 'inline':
				return 3;
		}

		return null;
	}

	private function get_decoded_conv_type( $type ) {
		switch ( $type ) {
			case 0:
				return __( 'Popup', 'optin' );
			case 1:
				return __( 'Banner', 'optin' );
			case 2:
				return __( 'Floating', 'optin' );
			case 3:
				return __( 'Inline', 'optin' );
			default:
				return null;
		}
	}

	private function get_decoded_integration_type( $type ) {
		switch ( $type ) {
			case 1:
				return 'FluentCRM';
			case 0:
			default:
				return 'Web Hook';
		}
	}

	private function get_encoded_traffic_channel( $data ) {

		/*
		Bit mapping:
		---------------

			0 0 0 0
			^ ^ ^ ^
			| | | |
			| | | -- org
			| | ---- paid
			| ------ social
			-------- direct
		*/

		if ( $data === 'default' || ( isset( $data->tSrc ) && $data->tSrc === 'all' ) ) {
			return 15;
		}

		if ( is_array( $data ) ) {

			$channel_lookup = array(
				'direct' => 8,
				'social' => 4,
				'paid'   => 2,
				'org'    => 1,
			);

			$curr_channel = 0;

			foreach ( $data as $channel ) {
				if ( isset( $channel_lookup[ $channel ] ) ) {
					$curr_channel |= $channel_lookup[ $channel ];
				}
			}

			return $curr_channel !== 0 ? $curr_channel : 15;
		}

		if ( $data->tSrc === 'channel' ) {
			return Helper::bin_array_to_dec(
				array(
					$data->tDirect,
					$data->tSocial,
					$data->tPaidSearch,
					$data->tOrgSearch,
				)
			);
		}

		return null;
	}

	private function get_encoded_visitor( $data ) {

		/*
			Bit mapping:
			---------------

			0 0 0 0
			^ ^ ^ ^
			| | | |
			| | | -- logged_out
			| | ---- logged_in
			| ------ returning
			-------- new
		*/

		if ( $data === 'default' || ( isset( $data->visitorType ) && $data->visitorType === 'all' ) ) {
			return 15;
		}

		if ( is_array( $data ) ) {

			$visitor_type_lookup = array(
				'new'        => 8,
				'returning'  => 4,
				'logged_in'  => 2,
				'logged_out' => 1,
			);

			$curr_types = 0;

			foreach ( $data as $type ) {
				if ( isset( $visitor_type_lookup[ $type ] ) ) {
					$curr_types |= $visitor_type_lookup[ $type ];
				}
			}

			return $curr_types;
		}

		return Helper::bin_array_to_dec(
			array(
				$data->visitorNew,
				$data->visitorRet,
				$data->visitorLoggedIn,
				$data->visitorLoggedOut,
			)
		);
	}

	private function get_encoded_device( $data ) {
		/*
			Bit mapping:
			---------------

			0 0 0
			^ ^ ^
			| | |
			| | -- lg
			| - -- sm
			- - -- xs
		*/

		if ( $data === 'default' || ( isset( $data->device ) && $data->device === 'all' ) ) {
			return 7;
		}

		if ( is_string( $data ) ) {
			$device_lookup = array(
				'xs' => 4,
				'sm' => 2,
				'lg' => 1,
			);
			return $device_lookup[ $data ] ?? 7;
		}

		return Helper::bin_array_to_dec(
			array(
				$data->xsDevice,
				$data->smDevice,
				$data->lgDevice,
			)
		);
	}

	private function get_encoded_os( $data ) {

		/*
			Bit mapping:
			---------------

			0 0 0 0 0
			^ ^ ^ ^ ^
			| | | | |
			| | | | - Windows
			| | | --- Mac
			| | ----- Linux
			| ------- Android
			--------- iOS
		*/

		if ( $data === 'default' || ( isset( $data->osType ) && $data->osType === 'all' ) ) {
			return 31;
		}

		if ( is_string( $data ) ) {
			$os_lookup = array(
				'Windows' => 1,
				'Mac'     => 2,
				'Linux'   => 4,
				'Android' => 8,
				'iOS'     => 16,
			);

			return $os_lookup[ $data ] ?? 31;
		}

			return Helper::bin_array_to_dec(
				array(
					$data->osIos,
					$data->osAndroid,
					$data->osLinux,
					$data->osMac,
					$data->osWindows,
				)
			);
	}

	private function get_encoded_integration( $int ) {
		switch ( $int ) {
			case 'fluentcrm':
				return 1;
			case 'webhook':
			default:
				return 0;
		}
	}

	private function get_cond( $conds, $cond = 'and' ) {

		$c = $cond === 'and' ? 'AND' : 'OR';

		if ( count( $conds ) < 1 ) {
			return ' ';
		}

		return implode( " {$c} ", $conds ) . ' ';
	}
	private function get_formatted_date( $date_str ) {
		$datetime = new \DateTime( $date_str );
		return $datetime->format( 'M j, Y' );
	}
}
