<?php

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    optin
 * @subpackage optin/includes
 */
class Init {


	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      OPTN_Loader $loader Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string $plugin_name The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string $version The current version of the plugin.
	 */
	protected $version;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function __construct() {
		$this->version     = OPTN_VERSION;
		$this->plugin_name = 'optin';

		$this->load_dependencies();
		$this->set_locale();
		$this->define_admin_hooks();
		$this->define_public_hooks();
	}


	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - OPTN_Loader. Orchestrates the hooks of the plugin.
	 * - OPTN_i18n. Defines internationalization functionality.
	 * - OPTN_Admin. Defines all hooks for the admin area.
	 * - OPTN_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies() {

		require_once OPTN_DIR . 'includes/class-wowoptin-helper.php';
		require_once OPTN_DIR . 'includes/trait-wowoptin-singleton.php';

		require_once OPTN_DIR . 'includes/class-detect-device.php';
		require_once OPTN_DIR . 'includes/class-wowoptin-country-decoder.php';
		require_once OPTN_DIR . 'includes/class-detect-os.php';
		require_once OPTN_DIR . 'includes/class-detect-traffic-channel.php';

		require_once OPTN_DIR . 'includes/class-wowoptin-db.php';

		require_once OPTN_DIR . 'includes/class-wowoptin-analytics.php';

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once OPTN_DIR . 'includes/class-wowoptin-loader.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		require_once OPTN_DIR . 'includes/class-wowoptin-i18n.php';

		/**
		 * The class responsible for defining all actions that occur in the admin area.
		 */
		require_once OPTN_DIR . 'admin/class-wowoptin-display-rules.php';
		require_once OPTN_DIR . 'admin/class-wowoptin-rest-admin.php';
		require_once OPTN_DIR . 'admin/class-wowoptin-templates.php';
		require_once OPTN_DIR . 'admin/class-wowoptin-admin.php';

		/**
		 * The class responsible for defining all actions that occur in the public-facing
		 * side of the site.
		 */
		require_once OPTN_DIR . 'public/class-visitor-count.php';
		require_once OPTN_DIR . 'public/class-wowoptin-conv-render.php';
		require_once OPTN_DIR . 'public/class-wowoptin-sanitizer.php';
		require_once OPTN_DIR . 'public/class-wowoptin-public.php';
		require_once OPTN_DIR . 'public/class-wowoptin-utils-public.php';
		require_once OPTN_DIR . 'public/class-wowoptin-rest-public.php';

		$this->loader = new OPTN_Loader();
	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the OPTN_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function set_locale() {

		$plugin_i18n = new OPTN_i18n();

		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );
	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_admin_hooks() {

		$plugin_admin = new \OPTN\Admin( $this->get_plugin_name(), $this->get_version() );

		// Admin customization.
		$this->loader->add_action( 'admin_menu', $plugin_admin, 'add_menu_page' );
		$this->loader->add_action( 'admin_footer', $plugin_admin, 'add_dropdown_html' );

		// Register REST routes.
		$this->loader->add_action( 'rest_api_init', $plugin_admin, 'register_routes' );

		// Admin panel scripts and styles.
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles' );
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts' );
	}

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_public_hooks() {

		$plugin_public = new OPTN_Public( $this->get_plugin_name(), $this->get_version() );

		// REST
		$this->loader->add_action( 'rest_api_init', $plugin_public, 'register_routes' );

		// Visitor count
		$this->loader->add_action( 'wp', $plugin_public, 'track_visit' );

		// Conversion content.
		$this->loader->add_action( 'safe_style_css', $plugin_public, 'add_to_style_allowlist' );
		$this->loader->add_action( 'wp', $plugin_public, 'get_conversions' );
		$this->loader->add_action( 'wp_head', $plugin_public, 'preconnect_google_fonts', 0, 0 );
		$this->loader->add_action( 'wp_footer', $plugin_public, 'add_html' );

		// Sales tracking.
		$this->loader->add_action( 'edd_complete_purchase', $plugin_public, 'edd_purchase_tracking' );
		$this->loader->add_action( 'woocommerce_thankyou', $plugin_public, 'woo_purchase_tracking' );

		// Enqueue scripts and styles
		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_styles', 999, 1 );
		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_scripts', 999, 1 );
	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function run() {
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @return    string    The name of the plugin.
	 * @since     1.0.0
	 */
	public function get_plugin_name() {
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @return    OPTN_Loader    Orchestrates the hooks of the plugin.
	 * @since     1.0.0
	 */
	public function get_loader() {
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @return    string    The version number of the plugin.
	 * @since     1.0.0
	 */
	public function get_version() {
		return $this->version;
	}
}
