<?php

	namespace OPTN;

	use OPTN\Helper;

	/**
	 * Renders conversion
	 *
	 * @link       https://wpxpo.com
	 * @since      1.0.0
	 *
	 * @package    optin
	 * @subpackage optin/public
	 */

	/**
	 * Renders Conversion
	 *
	 * @package    optin
	 * @subpackage optin/public
	 */
class ConvRender {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string $plugin_name The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string $version The current version of this plugin.
	 */
	private $version;


	private $fonts = array();

	/**
	 * Initialize the class and set its properties.
	 *
	 * @param string $plugin_name The name of the plugin.
	 * @param string $version The version of this plugin.
	 *
	 * @since    1.0.0
	 */
	public function __construct( $plugin_name, $version ) {
		$this->plugin_name = $plugin_name;
		$this->version     = $version;
	}

	public function render_inline_conversion( $post ) {
		$html  = '<div class="optn-inline-conversion" data-optn-post-id="' . esc_attr( $post->id ) . '"  data-optn-updated-at="' . esc_attr( $post->updated_at ) . '">';
		$html .= $this->get_conversion_item( $post );
		$html .= '</div>';

		$html .= '<div class="optn-inline-slot optn-conv-wrapper" id="optn-inline-slot-' . esc_attr( $post->id ) . '"></div>';

		return $html;
	}

	public function render_conversion( $posts ) {

		$html = '<div class="optn-conversion-contents" aria-hidden="true">';

		foreach ( $posts as $post ) {
			$html .= '<div class="optn-conversion-item" data-optn-post-id="' . esc_attr( $post->id ) . '" data-optn-optn-type="' . esc_attr( $post->type ) . '" data-optn-updated-at="' . esc_attr( $post->updated_at ) . '">';
			$html .= $this->get_conversion_item( $post );
			$html .= '</div>';
		}

		$html .= '</div>';

		add_action(
			'optn_html',
			function ( $html_str ) use ( $html ) {
				$html_str .= $html;
				return $html_str;
			}
		);
	}

	public function add_close_btn_content( $filename ) {
		add_action(
			'optn_data',
			function ( $data ) use ( $filename ) {
				if ( ! isset( $data[ $filename ] ) ) {
					$data[ $filename ] = file_get_contents( OPTN_DIR . '/assets/images/close-icons/' . $filename . '.svg' ); // phpcs:ignore WordPress.WP.AlternativeFunctions.file_get_contents_file_get_contents
				}
				return $data;
			}
		);
	}

	public function get_conversion_item( $post ) {
		if ( empty( $post ) ) {
			return '';
		}

		$post_id   = $post->id;
		$post_data = json_decode( $post->data );

		$steps = isset( $post_data->steps ) ? $post_data->steps : array();
		$html  = '';

		$this->add_close_btn_content( $post_data->design->close->icon );

		add_filter(
			'optn_data',
			function ( $data ) use ( $post_data, $post_id ) {
				$data[ $post_id ] = array(
					'design'       => $post_data->design,
					'schedule'     => $post_data->schedule,
					'audience'     => $post_data->audience,
					'goal'         => $post_data->goal,
					'trigger'      => $post_data->trigger,
					'displayRules' => $post_data->displayRules,
				);
				return $data;
			},
			9999,
			1
		);

		$css = $post_data->css ?? '';
		add_filter(
			'optn_css',
			function ( $css_str ) use ( $css ) {
				$css_str .= $css;
				return $css_str;
			}
		);

		if ( count( $steps ) > 0 ) {
			foreach ( $steps as $step ) {
				$html  .= '<div class="optn-conversion-step" data-step-id="' . esc_attr( $step->id ) . '">';
				$blocks = $step->blocks->value;
				foreach ( $blocks as $block ) {
					$html .= $this->render_block( $block, $post_data );
				}
				$html .= '</div>';
			}
		}

		return $html;
	}

	public function render_block( $block, $post_data = null ) {
		$method_name = '';

		if ( str_contains( $block[1]->name, '/' ) ) {
			[$parent]    = explode( '/', $block[1]->name );
			$method_name = $parent . '_block';
		} else {
			$method_name = $block[1]->name . '_block';
		}

		$html = '';

		if ( method_exists( $this, $method_name ) ) {
			add_filter(
				'optn_block_attrs',
				function ( $attrs ) use ( $block ) {
					$attrs[ $block[1]->clientId ] = $block[1]->attributes;
					return $attrs;
				}
			);
			$html = call_user_func( array( $this, $method_name ), $block[1], $post_data );
		} else {
			Helper::log( "Method $method_name does not exist" );
		}

		add_action(
			'optn_fonts',
			function () {
				foreach ( $this->fonts as $font_name => $weights ) {
					if ( ! empty( $font_name ) && ! empty( $weights ) ) {
						$weight_variations = array();

						// Sort weights numerically
						sort( $weights, SORT_NUMERIC );

						// Create sorted variations for both normal and italic styles
						foreach ( $weights as $weight ) {
							$weight = trim( $weight );
							if ( $weight !== '' ) {
								$weight_variations[] = "0,{$weight}";
							}
						}

						foreach ( $weights as $weight ) {
							$weight = trim( $weight );
							if ( $weight !== '' ) {
								$weight_variations[] = "1,{$weight}";
							}
						}

						if ( ! empty( $weight_variations ) ) {
							// Replace + with space in font name for the URL
							$font_name_url = str_replace( '+', ' ', $font_name );

							$font_url = 'https://fonts.googleapis.com/css2?family=' .
								urlencode( $font_name_url ) . ':ital,wght@' . implode( ';', $weight_variations ) .
								'&display=swap';

							// Use original font name with + for the handle
							$handle = sanitize_title( $this->plugin_name . '-font-' . $font_name );

							Helper::log_to_file( $font_url );

							wp_enqueue_style(
								$handle,
								esc_url( $font_url ),
								array(),
								OPTN_VERSION
							);
						}
					}
				}
			}
		);

		return $html;
	}

	// -----------------
	// Utilities
	// -----------------

	private function get_icon_html( $icon_name ) {
		$html = '';

		if ( empty( $icon_name ) ) {
			return $html;
		}

		$path = OPTN_DIR . '/assets/images/icon-library/' . $icon_name . '.svg';

		if ( file_exists( $path ) ) {
			$html = file_get_contents( $path ); // phpcs:ignore WordPress.WP.AlternativeFunctions.file_get_contents_file_get_contents
		}

		return $html;
	}

	private function add_google_font( $font_name, $weights ) {
		$sanitized_font_name = str_replace( ' ', '+', sanitize_text_field( $font_name ) );

		$sanitized_weights = array();

		foreach ( $weights as $weigth ) {
			$weigth = intval( $weigth );
			if ( $weigth >= 100 && $weigth <= 900 ) {
				$sanitized_weights[] = $weigth;
			}
		}

		if ( in_array( $sanitized_font_name, $this->fonts ) && ! empty( $sanitized_weights ) ) {
			foreach ( $sanitized_weights as $sweight ) {
				$this->fonts[ $sanitized_font_name ][] = $sweight;
			}
		} else {
			$this->fonts[ $sanitized_font_name ] = $sanitized_weights;
		}
	}

	private function get_link_opening_tag_html( $link, $attr ) {
		$html = '<a  href="' . esc_url( $link->url ) . '" target="' . esc_attr( $link->target ) . '" ' . $attr . ' ';

		if ( ! $link->doFollow ) {
			$html .= 'rel="nofollow" ';
		} elseif ( $link->sponsored ) {
			$html .= 'rel="sponsored" ';
		}

		if ( $link->download ) {
			$html .= 'download ';
		}

		$html .= ' >';

		return $html;
	}

	private function get_countdown_time_part_html( $def_value, $label, $type ) {
		$html  = '';
		$html .= '<div class="optn-timepart optn-timepart-' . esc_attr( $type ) . '">';

		$html .= '<div class="optn-timepart-digit optn-countdown-typo">';

		$values = str_split( $def_value );

		foreach ( $values as $value ) {
			$html .= '<span class="optn-timepart-digit-num">' . esc_html( $value ) . '</span>';
		}

		$html .= '</div>';

		$html .= '<div class="optn-timepart-label">' . esc_html( $label ) . '</div>';
		$html .= '</div>';
		return $html;
	}

	private function get_countdown_div_html( $div ) {
		return '<div class="optn-timepart-div">' . esc_html( $div ) . '</div>';
	}


	private function get_form_input_html( $field ) {
		$html  = '<label class="optn-block-form-input-label" for="' . esc_attr( $field->name ) . '" >';
		$html .= esc_html( $field->label );
		$html .= '</label>';

		$html .= '<div class="optn-block-form-input-wrapper" >';

		$html .= '<input class="optn-block-form-input-input optn-block-form-input-input-' . esc_attr( $field->id ) . '"';
		$html .= ' name="' . esc_attr( $field->name ) . '"';
		$html .= ' placeholder="' . esc_attr( $field->placeholder ) . '"';
		if ( $field->required ) {
			$html .= ' required ';
		}
		$html .= ' type="' . esc_attr( $field->inputType ) . '"';
		$html .= ' id="' . esc_attr( $field->id ) . '"';
		$html .= ' />';

		if ( ! empty( $field->message ) ) {
			$html .= '<div class="optn-block-form-input-msg">';
			$html .= esc_html( $field->message );
			$html .= '</div>';
		}

		$html .= '</div>';

		return $html;
	}

	private function get_form_fields_html( $fields ) {
		$html = '';

		foreach ( $fields as $field ) {
			$html .= '<div class="optn-block-form-input optn-block-form-input--' . esc_attr( $field->widthType ) . '">';
			$html .= $this->get_form_input_html( $field );
			$html .= '</div>';
		}

		return $html;
	}

	private function get_form_grouped_fields( $fields ) {

		$grouped_fields = array();
		$idx            = 0;

		while ( $idx < count( $fields ) ) {
			if ( $fields[ $idx ]->widthType === 'full' ) {
				$grouped_fields[] = array(
					$fields[ $idx ],
				);
				++$idx;
			} elseif ( $fields[ $idx ]->widthType !== 'full' && $idx !== count( $fields ) - 1 && $fields[ $idx + 1 ]->widthType !== 'full' ) {
				$grouped_fields[] = array(
					$fields[ $idx ],
					$fields[ $idx + 1 ],
				);
				$idx             += 2;

			} else {
				$grouped_fields[] =
					array(
						$fields[ $idx ],
					);

				$idx += 1;
			}
		}

		return $grouped_fields;
	}


	private function get_social_item( $icon, $label, $url ) {
		$html = '<a class="optn-block-social-content-item" href="' . esc_url( $url ) . '" >';

		$html .= file_get_contents( OPTN_DIR . '/assets/images/social-icons/' . $icon . '.svg' ); // phpcs:ignore WordPress.WP.AlternativeFunctions.file_get_contents_file_get_contents

		if ( ! empty( $label ) ) {
			$html .= '<span class="optn-block-social-content-item-label">';
			$html .= esc_html( $label );
			$html .= '</span>';
		}

		$html .= '</a>';

		return $html;
	}

	public function generateShareLink( $platform, $url ) {
		$encodedUrl = urlencode( sanitize_url( $url ) );

		switch ( $platform ) {
			case 'facebook':
				return "https://www.facebook.com/sharer/sharer.php?u=$encodedUrl";
			case 'instagram':
				return "https://www.instagram.com/?url=$encodedUrl";
			case 'linkedin':
				return "https://www.linkedin.com/shareArticle?mini=true&url=$encodedUrl";
			case 'discord':
				return "https://discord.com/channels/@me?url=$encodedUrl";
			case 'x':
				return "https://twitter.com/intent/tweet?url=$encodedUrl";
			default:
				return '';
		}
	}

	public function get_resp_hide_classes( &$attr ) {
		$cls = array();

		if ( isset( $attr->hideDesktop ) && $attr->hideDesktop ) {
			$cls[] = 'optn-hide-desktop';
		}

		if ( isset( $attr->hideTab ) && $attr->hideTab ) {
			$cls[] = 'optn-hide-tablet';
		}

		if ( isset( $attr->hideMobile ) && $attr->hideMobile ) {
			$cls[] = 'optn-hide-mobile';
		}

		return count( $cls ) < 1 ? '' : ' ' . implode( ' ', $cls ) . ' ';
	}

	public function get_watermark_html() {
		$html = '';

		$path = OPTN_DIR . '/assets/images/watermark.svg';

		if ( file_exists( $path ) ) {
			$html .= '<a href="https://www.wowoptin.com" target="_blank" style="text-decoration:none;position:absolute;bottom:0px;right:0px;display:flex;align-items:center;justify-content:center;">';
			$html .= file_get_contents( $path ); // phpcs:ignore WordPress.WP.AlternativeFunctions.file_get_contents_file_get_contents
			$html .= '</a>';

		}

		return $html;
	}

	// -----------------
	// Blocks
	// -----------------

	public function columns_block( $data, $post_data ) {
		$attr         = $data->attributes;
		$client_id    = $data->clientId;
		$inner_blocks = $data->innerBlocks->value;
		$bg           = $post_data->design->bg;

		$html  = '<div class="optn-block optn-block-columns optn-' . esc_attr( $client_id ) . '">';
		$html .= "<div class='optn-block-columns-content'>";

		if ( $bg->type === 'vid' ) {
			$url    = isset( $bg->vid->thumbSrc->id ) ? wp_get_attachment_image_url( $bg->vid->thumbSrc->id, 'full' ) : $bg->vid->thumbSrc->url;
			$repeat = $bg->vid->repeat === 'repeat' ? 'loop' : '';

			$html .= '<video
					class="optn-bg-video"
					' . esc_attr( $repeat ) . '
					src="' . esc_url( $bg->vid->url ) . '"
					poster="' . esc_url( $url ) . '"
					preload="auto"
					autoplay
					playsinline
					muted
				></video>';
		}

		if ( ! empty( $inner_blocks ) ) {
			foreach ( $inner_blocks as $block ) {
				$html .= $this->render_block( $block );
			}
		}

		$html .= '</div>';

		if ( defined( 'OPTN_SHOW_WATERMARK' ) && true === OPTN_SHOW_WATERMARK ) {
			$html .= $this->get_watermark_html();
		}

		$html .= '</div>';

		return $html;
	}

	public function column_block( $data ) {
		$attr         = $data->attributes;
		$client_id    = $data->clientId;
		$inner_blocks = isset( $data->innerBlocks ) ? $data->innerBlocks->value : null;

		$html  = '<div class="optn-block optn-block-column optn-' . esc_attr( $client_id ) . '" data-optn-block-index="' . esc_attr( $attr->index ) . '">';
		$html .= '<div class="optn-block-column-content" >';

		if ( ! empty( $inner_blocks ) ) {
			foreach ( $inner_blocks as $block ) {
				$html .= $this->render_block( $block );
			}
		}

		$html .= '</div>';
		$html .= '</div>';

		return $html;
	}

	public function container_block( $data ) {
		$attr         = $data->attributes;
		$client_id    = $data->clientId;
		$inner_blocks = isset( $data->innerBlocks ) ? $data->innerBlocks->value : null;

		$resp_cls = $this->get_resp_hide_classes( $attr );
		$html     = '<div class="optn-block optn-block-container optn-' . esc_attr( $client_id ) . $resp_cls . '">';
		$html    .= '<div class="optn-block-container-content">';

		if ( ! empty( $inner_blocks ) ) {
			foreach ( $inner_blocks as $block ) {
				$html .= $this->render_block( $block );
			}
		}

		$html .= '</div>';
		$html .= '</div>';

		return $html;
	}

	public function form_block( $data ) {
		$attr         = $data->attributes;
		$client_id    = $data->clientId;
		$inner_blocks = isset( $data->innerBlocks ) ? $data->innerBlocks->value : null;

		$this->add_google_font( $attr->labelTypo->fontFamily, array( $attr->labelTypo->fontWeight ) );
		$this->add_google_font( $attr->inputTypo->fontFamily, array( $attr->inputTypo->fontWeight ) );
		$this->add_google_font( $attr->msgTypo->fontFamily, array( $attr->msgTypo->fontWeight ) );

		$layout = $attr->layout;

		$html  = '<form class="optn-block optn-block-form optn-' . esc_attr( $client_id ) . '" id="' . esc_attr( $client_id ) . '">';
		$html .= '<div class="optn-block-form-content">';
		$html .= '<div class="optn-block-form-fields">';

		if ( $layout === '2col' ) {
			$grouped_fields = $this->get_form_grouped_fields( $attr->fields );

			foreach ( $grouped_fields as $fields ) {
				$html .= '<div class="optn-block-form-field-group">';
				$html .= $this->get_form_fields_html( $fields );
				$html .= '</div>';
			}
		} else {
			$html .= $this->get_form_fields_html( $attr->fields );
		}

		$html .= '</div>';

		if ( ! empty( $inner_blocks ) ) {
			foreach ( $inner_blocks as $block ) {
				$html .= $this->render_block( $block );
			}
		}

		$html .= '</div>';
		$html .= '</form>';

		return $html;
	}

	public function heading_block( $data ) {
		$attr      = $data->attributes;
		$client_id = $data->clientId;

		$fonts = array_unique(
			array(
				$attr->typo->fontWeight,
				$attr->typo->boldFontWeight,
			)
		);
		$this->add_google_font( $attr->typo->fontFamily, $fonts );

		$resp_cls = $this->get_resp_hide_classes( $attr );
		$html     = '<div class="optn-block optn-block-heading optn-' . esc_attr( $client_id ) . $resp_cls . '">';
		$html    .= $attr->text;
		$html    .= '</div>';

		return $html;
	}

	public function countdown_block( $data ) {
		$attr      = $data->attributes;
		$client_id = $data->clientId;

		$this->add_google_font( $attr->digitTypo->fontFamily, array( $attr->digitTypo->fontWeight ) );

		$this->add_google_font( $attr->labelTypo->fontFamily, array( $attr->labelTypo->fontWeight ) );

		$def_value = $attr->format00 ? '00' : '0';

		$resp_cls = $this->get_resp_hide_classes( $attr );
		$html     = '<div class="optn-block optn-block-countdown optn-' . esc_attr( $client_id . $resp_cls ) . '" id="' . esc_attr( $client_id ) . '">';

		$html .= '<div style="display: none;" class="optn-countdown-typo optn-block-countdown-replace-text" id="optn-countdown-replace-text-' . esc_attr( $client_id ) . '"><span>' . esc_html( $attr->endMsg ) . '</span></div>';

		$html .= '<div class="optn-block-countdown-content">';

		if ( $attr->dayEnabled ) {
			$html .= $this->get_countdown_time_part_html( $def_value, $attr->dayLabel, 'day' );
		}

		if ( $attr->dayEnabled && $attr->enableSep ) {
			$html .= $this->get_countdown_div_html( $attr->sepStyle );
		}

		if ( $attr->hourEnabled ) {
			$html .= $this->get_countdown_time_part_html( $def_value, $attr->hourLabel, 'hour' );
		}

		if ( $attr->hourEnabled && $attr->enableSep ) {
			$html .= $this->get_countdown_div_html( $attr->sepStyle );
		}

		if ( $attr->minEnabled ) {
			$html .= $this->get_countdown_time_part_html( $def_value, $attr->minLabel, 'min' );
		}

		if ( $attr->minEnabled && $attr->enableSep ) {
			$html .= $this->get_countdown_div_html( $attr->sepStyle );
		}

		if ( $attr->secEnabled ) {
			$html .= $this->get_countdown_time_part_html( $def_value, $attr->secLabel, 'sec' );
		}

		$html .= '</div>';
		$html .= '</div>';

		return $html;
	}

	public function separator_block( $data ) {
		$attr      = $data->attributes;
		$client_id = $data->clientId;

		$resp_cls = $this->get_resp_hide_classes( $attr );
		$html     = '<div class="optn-block optn-block-separator optn-' . esc_attr( $client_id ) . $resp_cls . '">';

		if ( $attr->style === 'simple' ) {
			$html .= '<hr class="optn-block-separator-simple" />';
		} else {
			$html .= '<hr class="optn-block-separator-simple" />';

				$html .= '<div class="optn-block-separator-content">';

			if ( $attr->style === 'text' ) {

				$fonts = array_unique(
					array(
						$attr->typo->fontWeight,
						$attr->typo->boldFontWeight,
					)
				);
				$this->add_google_font( $attr->typo->fontFamily, $fonts );

				$html .= $attr->text;
			} else {
				$html .= $this->get_icon_html( $attr->icon );
			}

			$html .= '</div>';

			$html .= '<hr class="optn-block-separator-simple" />';
		}

		$html .= '</div>';

		return $html;
	}

	public function image_block( $data ) {
		$attr      = $data->attributes;
		$client_id = $data->clientId;

		$resp_cls = $this->get_resp_hide_classes( $attr );
		$html     = '<div class="optn-block optn-block-image optn-' . esc_attr( $client_id ) . $resp_cls . '">';

		if ( ! empty( $attr->link->url ) ) {
			$html .= $this->get_link_opening_tag_html( $attr->link, 'class="optn-block-image-content"' );
		} else {
			$html .= '<div class="optn-block-image-content" >';
		}

		$html .= '<img src="' . esc_url( $attr->src->url ) . '" alt="' . esc_attr( $attr->alt ) . '" />';

		$html .= ! empty( $attr->link->url ) ? '</a>' : '</div>';
		$html .= '</div>';

		return $html;
	}

	public function video_block( $data ) {
		$attr      = $data->attributes;
		$client_id = $data->clientId;

		$resp_cls = $this->get_resp_hide_classes( $attr );
		$html     = '<div class="optn-block optn-block-video optn-' . esc_attr( $client_id ) . $resp_cls . '">';
		$html    .= '<div class="optn-block-video-content">';

		$thumb_url = isset( $attr->thumbSrc->id ) ? wp_get_attachment_image_url( $attr->thumbSrc->id, 'full' ) : $attr->thumbSrc->url;
		$repeat    = $attr->repeat === 'repeat' ? 'loop' : '';
		$autoplay  = $attr->autoplay ? 'autoplay muted data-optn-autoplay="true"' : '';
		$controls  = $attr->controls ? 'controls' : '';

		$html .= '
			<video
				' . esc_attr( $repeat ) . '
				src="' . esc_url( $attr->url ) . '
				poster="' . esc_url( $thumb_url ) . '"
				preload="auto"
				' . esc_attr( $controls ) . '
				' . esc_attr( $autoplay ) . '
				aria-description="' . esc_attr( $attr->desc ) . '"
			></video>';

		$html .= '</div>';
		$html .= '</div>';

		return $html;
	}

	public function spacer_block( $data ) {
		// $attr      = $data->attributes;
		$client_id = $data->clientId;

		$resp_cls = $this->get_resp_hide_classes( $attr );
		$html     = '<div class="optn-block-spacer optn-' . esc_attr( $client_id ) . $resp_cls . '"></div>';

		return $html;
	}

	public function button_block( $data ) {
		$attr      = $data->attributes;
		$client_id = $data->clientId;

		$fonts = array_unique(
			array(
				$attr->typo->fontWeight,
				$attr->typo->boldFontWeight,
			)
		);

		$this->add_google_font( $attr->typo->fontFamily, $fonts );

		$resp_cls = $this->get_resp_hide_classes( $attr );
		$iconHtml = ! empty( $attr->icon ) ? $this->get_icon_html( $attr->icon ) : '';
		$html     = '<div class="optn-block optn-block-button optn-' . esc_attr( $client_id ) . $resp_cls . '" id="' . esc_attr( $client_id ) . '">';

		if ( $attr->action === 'link' && ! $attr->isSubmit ) {
			$html .= $this->get_link_opening_tag_html( $attr->link, 'class="optn-block-button-btn"' );
		} elseif ( $attr->action === 'call' ) {
			$html .= '<a class="optn-block-button-btn" href="tel:' . esc_attr( $attr->phone ) . '" />';
		} else {

			$escaped_submit_data = '';

			if ( $attr->isSubmit ) {
				$escaped_submit_data =
					esc_attr(
						wp_json_encode(
							array(
								'text'           => $attr->text,
								'submittingText' => $attr->submittingText,
								'submittedText'  => $attr->submittedText,
							)
						)
					);
			}

			$html .= '<button class="optn-block-button-btn" type="' . ( $attr->isSubmit ? 'submit' : 'button' ) . '" data-optn="' . $escaped_submit_data . '">';
		}

		if ( $attr->iconPos === 'left' ) {
			$html .= $iconHtml;
		}

		$html .= '<div class="optn-block-button-btn-text">' . $attr->text . '</div>';
		// $html .= $attr->text;

		if ( $attr->iconPos === 'right' ) {
			$html .= $iconHtml;
		}

		$html .= ( $attr->action === 'link' || $attr->action === 'call' ) ? '</a>' : '</button>';

		$html .= '</div>';

		return $html;
	}

	public function html_block( $data ) {
		$attr      = $data->attributes;
		$client_id = $data->clientId;

		$resp_cls = $this->get_resp_hide_classes( $attr );
		$html     = '<div class="optn-block optn-block-html optn-' . esc_attr( $client_id ) . $resp_cls . '">';
		$html    .= '<div class="optn-block-html-content">';
		$html    .= wp_kses_post( $attr->html );
		$html    .= '</div>';
		$html    .= '</div>';

		return $html;
	}

	public function social_block( $data ) {
		$attr      = $data->attributes;
		$client_id = $data->clientId;

		$this->add_google_font( $attr->typo->fontFamily, array( $attr->typo->fontWeight ) );

		$resp_cls = $this->get_resp_hide_classes( $attr );
		$html     = '<div class="optn-block optn-block-social optn-' . esc_attr( $client_id ) . $resp_cls . '">';
		$html    .= '<div class="optn-block-social-content">';

		foreach ( $attr->socials as $social ) {
			$html .= $this->get_social_item(
				$social->type,
				$attr->labelEnabled && $social->labelEnabled ?
				$social->label : null,
				$attr->type === 'follow' ? $social->url : $this->generateShareLink( $social->type, $social->url ),
			);
		}

		$html .= '</div>';
		$html .= '</div>';

		return $html;
	}

	public function coupon_block( $data ) {
		$attr      = $data->attributes;
		$client_id = $data->clientId;

		$msg_fonts = array_unique(
			array(
				$attr->msgTypo->fontWeight,
				$attr->msgTypo->boldFontWeight,
			)
		);

		$ulabel_fonts = array_unique(
			array(
				$attr->uLabelTypo->fontWeight,
				$attr->uLabelTypo->boldFontWeight,
			)
		);

		$code_fonts = array_unique(
			array(
				$attr->codeTypo->fontWeight,
				$attr->codeTypo->boldFontWeight,
			)
		);

		$this->add_google_font( $attr->msgTypo->fontFamily, $msg_fonts );

		$this->add_google_font( $attr->uLabelTypo->fontFamily, $ulabel_fonts );

		$this->add_google_font( $attr->codeTypo->fontFamily, $code_fonts );

		$resp_cls = $this->get_resp_hide_classes( $attr );
		$html     = '<div class="optn-block optn-block-coupon optn-' . esc_attr( $client_id ) . $resp_cls . '" id="' . esc_attr( $client_id ) . '">';
		$html    .= '<div class="optn-block-coupon-content" >';

		$html .= '<button class="optn-block-coupon-code-btn optn-block-coupon-code-btn-preset-' . esc_attr( $attr->preset ) . '">';

		if ( $attr->preset === '1' ) {
			$html .= '<div class="optn-block-coupon-msg" >';
			$html .= $attr->msgText;
			$html .= '</div>';
		}

		$html .= '<div class="optn-block-coupon-code" >';
		$html .= $attr->code;
		$html .= '</div>';

		if ( $attr->preset === '2' ) {
			$html .= $this->get_icon_html( 'icon_34' );
		}

		$html .= '</button>';

		if ( $attr->type === 'reveal' ) {
			$html .= '<button class="optn-block-coupon-ubtn">';

			if ( $attr->iconPos === 'left' ) {
				$html .= $this->get_icon_html( $attr->icon );
			}

			$html .= $attr->uLabel;

			if ( $attr->iconPos === 'right' ) {
				$html .= $this->get_icon_html( $attr->icon );
			}

			$html .= '</button>';
		}

		$html .= '</div>';
		$html .= '</div>';

		return $html;
	}

	public function shortcode_block( $data ) {
		$attr      = $data->attributes;
		$client_id = $data->clientId;
		$resp_cls  = $this->get_resp_hide_classes( $attr );

		$html = '<div class="optn-block optn-block-shortcode optn-' . esc_attr( $client_id . $resp_cls ) . '">';

		$html .= do_shortcode( $attr->code );

		$html .= '</div>';

		return $html;
	}
}
