<?php

use OPTN\Helper;
use OPTN\Sanitizer;
use OPTN\Utils;
use OPTN\VisitorCount;


	/**
	 * The public-facing functionality of the plugin.
	 *
	 * @link       https://wpxpo.com
	 * @since      1.0.0
	 *
	 * @package    optin
	 * @subpackage optin/public
	 */

	/**
	 * The public-facing functionality of the plugin.
	 *
	 * Defines the plugin name, version, and two examples hooks for how to
	 * enqueue the public-facing stylesheet and JavaScript.
	 *
	 * @package    optin
	 * @subpackage optin/public
	 */
class OPTN_Public {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string $plugin_name The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string $version The current version of this plugin.
	 */
	private $version;

	/**
	 * Conversion renderer
	 *
	 * @var \OPTN\ConvRender $renderer
	 */
	private $renderer;

	/**
	 * DB instance
	 *
	 * @var \OPTN\DB $db
	 */
	private $db;

	/**
	 * VisitorCount instance
	 *
	 * @var \OPTN\VisitorCount $visitor_count
	 */
	private $visitor_count;

	private $embed_data = array();


	// Conversion types
	const POPUP_CONVERSION        = 'popup';
	const BANNER_CONVERSION       = 'banner';
	const SLIDE_IN_CONVERSION     = 'slidein';
	const FLOATING_BAR_CONVERSION = 'floatingbar';

	private $user_info;

	private $should_enqueue = false;


	/**
	 * Initialize the class and set its properties.
	 *
	 * @param string $plugin_name The name of the plugin.
	 * @param string $version The version of this plugin.
	 *
	 * @since    1.0.0
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version     = $version;
		$this->renderer    = new \OPTN\ConvRender( $plugin_name, $version );
		$this->db          = \OPTN\DB::get_instance();
		$this->visitor_count = VisitorCount::get_instance();

		$this->visitor_count->schedule_rotation();

		add_shortcode( 'optn', array( $this, 'get_inline_conversion' ) );

		// TODO @samin: Needs API for data
		// add_action( 'wp', array( $this, 'setup_embedded_optins' ) );
	}

	public function register_routes() {
		\OPTN\Analytics::get_instance()->register_public_routes();
		new \OPTN\RestPublic();
	}

	public function track_visit() {
		$this->visitor_count->track_visit();
	}

	public function get_user_info() {

		if ( empty( $this->user_info ) ) {

			$post_id = get_the_ID();

			$this->user_info = array(
				'post_id'    => strval( $post_id ),
				'term_ids'   => Utils::get_terms_by_id( $post_id ),
				'front_page' => is_front_page(),
				'params'     => Utils::get_sanitized_query_parameters(),
				'author_id'  => strval( get_the_author_meta( 'ID' ) ),
				'post_type'  => get_post_type( $post_id ),

				'woo'        => Utils::get_woo_info(),
				'edd'        => Utils::get_edd_info(),
			);
		}

		return $this->user_info;
	}

	public function add_to_style_allowlist( $styles ) {
		return Sanitizer::add_to_style_allowlist( $styles );
	}

	public function preconnect_google_fonts() {
		echo '<link rel="preconnect" href="https://fonts.gstatic.com/" crossorigin>';
		echo '<link rel="preconnect" href="https://fonts.googleapis.com/" crossorigin>';
	}

	/**
	 * Adds optin html to the currrent page
	 *
	 * @return void
	 */
	public function add_html() {
		$html = apply_filters( 'optn_html', '' );
		echo \OPTN\Sanitizer::sanitize( $html );
	}

	public function setup_embedded_optins() {
		$posts = $this->db->get_conv_for_render( null, true );

		if ( empty( $posts ) ) {
			return;
		}

		foreach ( $posts as $post ) {
			if ( $this->check_display_rules( $post ) ) {

				$hook_data = Utils::get_embed_hook_and_pos( $post->embed );

				if ( empty( $hook_data['hook'] ) || empty( $hook_data['pos'] ) ) {
					continue;
				}

				if ( isset( $this->embed_data[ $hook_data['hook'] ] ) ) {
					$this->embed_data[ $hook_data['hook'] ][] = array(
						'post'   => $post,
						'pos'    => $hook_data['pos'],
						'single' => isset( $hook_data['single'] ) ? $hook_data['single'] : false,
					);
				} else {
					$this->embed_data[ $hook_data['hook'] ] = array(
						array(
							'post'   => $post,
							'pos'    => $hook_data['pos'],
							'single' => isset( $hook_data['single'] ) ? $hook_data['single'] : false,
						),
					);
				}

				add_filter( $hook_data['hook'], array( $this, 'get_embedded_conversion' ), 999 );
			}
		}
	}

	public function get_embedded_conversion( $content ) {

		$curr_hook = current_filter();

		if ( ! isset( $this->embed_data[ $curr_hook ] ) || ! is_array( $this->embed_data[ $curr_hook ] ) ) {
			return $content;
		}

		$data = $this->embed_data[ $curr_hook ];

		// if ( isset( $data['single'] ) && boolval( $data['single'] ) && is_single() ) {
		// return $content;
		// }

		$this->set_allow_enqueuing( true );

		foreach ( $data as $d ) {
			$html    = $this->renderer->render_inline_conversion( $d['post'] );
			$html    = \OPTN\Sanitizer::sanitize( $html );
			$content = 'before' === $d['pos'] ? $html . $content : $content . $html;
		}

		return $content;
	}

	public function get_inline_conversion( $sc_attr ) {

		if ( ! is_numeric( $sc_attr['id'] ) ) {
			return '';
		}

		$post = $this->db->get_conv_for_render( intval( $sc_attr['id'] ) );

		if ( empty( $post[0] ) || ! $this->check_display_rules( $post[0] ) ) {
			return '';
		}

		$this->set_allow_enqueuing( true );
		$html = $this->renderer->render_inline_conversion( $post[0] );

		return \OPTN\Sanitizer::sanitize( $html );
	}

	public function get_conversions() {

		$posts = $this->db->get_conv_for_render();

		if ( empty( $posts ) ) {
			return;
		}

		$filtered_posts = array();

		foreach ( $posts as $post ) {
			if ( $this->check_display_rules( $post ) ) {
				$filtered_posts[] = $post;
			}
		}

		if ( ! empty( $filtered_posts ) ) {
			$this->set_allow_enqueuing( true );
			$this->renderer->render_conversion( $filtered_posts );
		}
	}

	private function check_display_rules( $post ) {
		// Condition among groups -> OR
		// Condition among rule inside a group -> AND
		// So if one group is true, immediately return true (short-circuit),
		// and if one rule in a group is false, the whole group is false
		// We should sort the groups by ascending order in terms of number of rules in a group

		$user_info   = $this->get_user_info();
		$data        = json_decode( $post->data );
		$rule_groups = $data->displayRules;
		$passed      = false;

		usort(
			$rule_groups,
			function ( $a, $b ) {
				return count( $a->rules ) - count( $b->rules );
			}
		);

		foreach ( $rule_groups as $rule_group ) {
			$res_group = true;

			foreach ( $rule_group->rules as $rule ) {
				$type   = $rule->type;
				$values = $rule->values;
				$cond   = $rule->cond;

				// Entire Page
				if ( str_starts_with( $type, 'entire' ) ) {
					continue;
				} 
				
				// Front page
				elseif ( str_starts_with( $type, 'front' ) && ! $user_info['front_page'] ) {
					$res_group = false;
					break;
				} 
				
				// Single (with custom post types)
				elseif ( (
						$type === 'post_posts' ||
						$type === 'page' ||
						str_starts_with( $type, 'cpt_post:' )
					) &&
					! Utils::check_values( $user_info['post_id'], $values, $cond )
				) {
					$res_group = false;
					break;
				} 
				
				// Taxonomy (with custom taxonomy)
				elseif (
					(
						$type === 'post_cat' ||
						$type === 'post_tags' ||
						str_starts_with( $type, 'cpt_tax:' )
					) &&
					! Utils::check_values( $user_info['term_ids'], $values, $cond )
				) {
					$res_group = false;
					break;
				} 
				
				// Post Types	
				elseif (
					$type === 'post_type' &&
					! Utils::check_values( $user_info['post_type'], $values, $cond )
				) {
					$res_group = false;
					break;
				} 
				
				// Author
				elseif (
					$type === 'author' &&
					! Utils::check_values( $user_info['author'], $values, $cond )
				) {
					$res_group = false;
					break;
				}

				// Woo Cart Page
				elseif (
					$type === 'woo_cart_nocond' &&
					! $user_info['woo']['cart_page']
				) {
					$res_group = false;
					break;
				} 
				
				// Woo Thank You Page
				elseif (
					$type === 'woo_thank_nocond' &&
					! $user_info['woo']['thank_you_page']
				) {
					$res_group = false;
					break;
				} 
			
				// Woo Products in Cart
				elseif (
					$type === 'woo_cart_products' &&
					! Utils::check_values(
						$user_info['woo']['products'],
						$values,
						$cond,
						$user_info['woo']['all_count'],
					)
				) {
					$res_group = false;
					break;
				} 
				
				// Woo Number of products in Cart
				elseif (
					$type === 'woo_cart_products_num' &&
					! Utils::check_math( $user_info['woo']['num_of_products'], $values, $cond )
				) {
					$res_group = false;
					break;
				} 
				
				// Woo Cart Total Value
				elseif (
					$type === 'woo_cart_value$' &&
					! Utils::check_math( $user_info['woo']['total'], $values, $cond )
				) {
					$res_group = false;
					break;
				}

				// EDD Cart Page
				elseif (
					$type === 'edd_cart_nocond' &&
					! $user_info['edd']['cart_page']
				) {
					$res_group = false;
					break;
				} 
				
				// EDD Thank You Page
				elseif (
					$type === 'edd_thank_nocond' &&
					! $user_info['edd']['thank_you_page']
				) {
					$res_group = false;
					break;
				} 
				
				// EDD Products in Cart
				elseif (
					$type === 'edd_cart_products' &&
					! Utils::check_values(
						$user_info['edd']['products'],
						$values,
						$cond,
						$user_info['edd']['all_count'],
					)
				) {
					$res_group = false;
					break;
				} 
				
				// EDD Number of products in Cart
				elseif (
					$type === 'edd_cart_products_num' &&
					! Utils::check_math( $user_info['edd']['num_of_products'], $values, $cond )
				) {
					$res_group = false;
					break;
				} 
				
				// EDD Total Cart Value
				elseif (
					$type === 'edd_cart_value$' &&
					! Utils::check_math( $user_info['edd']['total'], $values, $cond )
				) {
					$res_group = false;
					break;
				}
			}

			if ( $res_group ) {
				$passed = true;
				break;
			} else {
				$res_group = true;
			}
		}

		return $passed;
	}


	public function edd_purchase_tracking( $payment_id ) {
		$this->track_purchase( 'edd', $payment_id );
	}

	public function woo_purchase_tracking( $order_id ) {
		$this->track_purchase( 'woo', $order_id );
	}

	private function track_purchase( $order_type, $order_id ) {
		$cookie = isset( $_COOKIE['optn_purchase_tracking'] ) ? sanitize_text_field( wp_unslash( $_COOKIE['optn_purchase_tracking'] ) ) : null;
		$data   = json_decode( stripslashes( $cookie ) );
		if ( ! empty( $data ) ) {
			$this->db->add_purchase( $data->analyticsId, $data->postId, $order_type, $order_id );
			setcookie( 'optn_purchase_tracking', '', strtotime( '-1 day' ) );
		}
	}

	/**
	 * Register the stylesheets for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {

		// if ( ! $this->get_allow_enqueuing() ) {
		// return;
		// }

		wp_enqueue_style( $this->plugin_name . '-public-css', OPTN_URL . 'public/css/wowoptin-public.min.css', array(), $this->version, 'all' );
		wp_add_inline_style( $this->plugin_name . '-public-css', wp_strip_all_tags( apply_filters( 'optn_css', '' ) ) );
	}


	/**
	 * Register the JavaScript for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {

		// if ( ! $this->get_allow_enqueuing() ) {
		// return;
		// }

		$assets = require_once OPTN_DIR . 'build/fe/index.asset.php';

		do_action( 'optn_fonts' );

		wp_enqueue_script(
			$this->plugin_name . '-public-js',
			OPTN_URL . 'build/fe/index.js',
			$assets['dependencies'],
			$assets['version'],
			array(
				'strategy' => 'defer',
			)
		);

		$attrs      = apply_filters( 'optn_block_attrs', array() );
		$data       = apply_filters( 'optn_data', array() );
		$assets_url = OPTN_URL . '/assets';

		wp_localize_script(
			$this->plugin_name . '-public-js',
			'optn',
			array(
				'url'        => admin_url( 'admin-ajax.php' ),
				'attrs'      => $attrs,
				'data'       => $data,
				'assets_url' => $assets_url,
				'nonce'      => wp_create_nonce( 'wp_rest' ),
			)
		);
	}

	/**
	 * Set enqueing
	 *
	 * @param boolean $value
	 * @return void
	 */
	private function set_allow_enqueuing( $value ) {
		$this->should_enqueue = is_bool( $value ) ? $value : false;
	}


	/**
	 * Get enqueuing
	 *
	 * @return boolean
	 */
	private function get_allow_enqueuing() {
		return $this->should_enqueue;
	}
}
